!> \file
!! Contains modules cl_con and conduction
!<

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Module CL_CON
!
!> Contains the variables used for the conduction boundary
!! conditions.
!<
module cl_con
  implicit none
  integer, dimension(6)              :: icl_con             !< Local  conduction boundary conditions
  integer, dimension(6)              :: icl_con_glob        !< Global conduction boundary conditions
  real   , dimension(:), allocatable :: T_l                 !< Temperature in the conduction boundaries
end module cl_con

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Module conduction
!
!> Contains the variables used for the thermal conduction.
!<
module conduction
  implicit none
  real    :: varrel_con        !< Maximum thermal conduction quantity variation allowed in an implicit timestep
  real    :: tolx_con          !< Thermal conduction convergence tolerance threshold
  real    :: vardt_con         !< Maximum thermal conduction timestep variation allowed
  real    :: dt_con_exp        !< Explicit thermal conduction timestep
  real    :: dt_con_imp        !< Implicit thermal conduction timestep
  real    :: dt_con            !< Thermal conduction timestep
  logical :: implicit_con      !< If .true. the conduction is computed using an implicit method
  integer :: nitetot_con       !< Conduction total number of iterations
  logical :: precond_con       !< Perform matrix preconditionning if .true.
end module conduction
