!> \file
!! Contains module gravity and subroutine
!! allocate_array_gra()
!<

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Module GRAVITY
!
!> Contains all the gravity variables
!<
module gravity
  implicit none
  integer                             :: gravity_type          !< Type of gravity: 1=self-gravity, 2=point source potential, 3=onstant gravitational field
  integer                             :: ISOLE                 !< Isolated computational domain?
  integer                             :: nitetot_grav          !< Total number of iteration in Poisson solver
  real, dimension(:,:,:), allocatable :: phi                   !< Gravitational potential \f$ \Phi \f$ 
  real, dimension(:,:,:), allocatable :: phiold                !< Old graviational potential \f$ \Phi_{\mathrm{old}} \f$
  real, dimension(4    )              :: gravity_params        !< Gravity parameters
end module gravity

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Subroutine ALLOCATE_ARRAY_GRA
!
!> Allocates the gravity arrays.
!<
subroutine allocate_array_gra

  use parameters
  use gravity

  implicit none
  
  allocate(phi   (nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)))
  allocate(phiold(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)))

  return
  
end subroutine allocate_array_gra
