!> \file
!! Contains subroutine pasdt_grav()
!<

!###########################################################
!###########################################################
!###########################################################
!###########################################################

! Subroutine PASDT_GRAV
!
!> Computes the minimum free-fall time in the grid
!! and calculates a free-fall limited timestep.
!<
subroutine pasdt_grav

  use divers, only : verbose,dt_ff
  use var   , only : rho
  use unites, only : G

  implicit None

#if PARA==1
  include 'mpif.h'
  integer :: info
#endif

  ! Local variables
  real :: dt_ff_loc
 
  if(verbose) write(*,*) 'Start of pasdt_grav'
 
  dt_ff_loc = 1.0/sqrt(G*maxval(rho))*0.01
        
#if PARA==1
  info = 0
  call MPI_ALLREDUCE(dt_ff_loc,dt_ff,1,MPI_DOUBLE_PRECISION,MPI_MIN,MPI_COMM_WORLD,info)
#else
  dt_ff = dt_ff_loc
#endif

  if(verbose) write(*,*) 'End of pasdt_grav'

  return
  
end Subroutine pasdt_grav
