!> \file
!! Contains subroutine update_gravity()
!<

!###########################################################
!###########################################################
!###########################################################
!###########################################################

! Subroutine UPDATE_GRAVITY
!
!> This routine includes the gravity source term in the rhou
!! and E hydro variables.
!<
subroutine update_gravity

  use var       , only : rho,rhou,E
  use varold    , only : rhoold,rhouold
  use geom      , only : x,Cylindrique,Spherique,geom_dir
  use parameters, only : nx,ndim
  use divers    , only : dt
  use gravity
  use const
  
  implicit none
  
  integer :: i,j,k

  real :: rhon,rhonp1,rhoun,rhounp1,dphin,dphinp1
  real :: xim1,xip1,yjm1,yjp1,zkm1,zkp1
  real :: dtdx,dtdy,dtdz,xc,sinxc

  ! Gravitational acceleration along first direction
  do k = 1,nx(3)
     do j = 1,nx(2)
        do i = 1,nx(1)
           
           xim1 = half*(x(i-1,1)+x(i  ,1))
           xip1 = half*(x(i+1,1)+x(i+2,1))
           dtdx = dt/(xip1-xim1)
           
           ! density
           rhon   = rhoold(i,j,k)
           rhonp1 = rho   (i,j,k)

           ! momentum
           rhoun   = rhouold(i,j,k,1)
           rhounp1 = rhou   (i,j,k,1)

           ! slope of phi along x direction
           dphin   = phiold(i+1,j,k)-phiold(i-1,j,k)
           dphinp1 = phi   (i+1,j,k)-phi   (i-1,j,k)
           
           rhou(i,j,k,1) = rhou(i,j,k,1) - half*(rhon *dphin+rhonp1 *dphinp1)*dtdx
           E   (i,j,k  ) = E   (i,j,k  ) - half*(rhoun*dphin+rhounp1*dphinp1)*dtdx
           

        end do
     end do
  end do

  ! Gravitational acceleration along second direction
  if (ndim >= 2) then
     do k = 1,nx(3)
        do j = 1,nx(2)
           do i = 1,nx(1)
             
              yjm1 = half*(x(j-1,2)+x(j  ,2))
              yjp1 = half*(x(j+1,2)+x(j+2,2))
              dtdy = dt/(yjp1-yjm1)
              xc   = 1.
              if ((geom_dir(2).eq.'phi_c').or.(Spherique)) xc = half*(x(i,1)+x(i+1,1))
              
              ! density
              rhonp1 = rho   (i,j,k)
              rhon   = rhoold(i,j,k)

              ! momentum
              rhoun   = rhouold(i,j,k,2)
              rhounp1 = rhou   (i,j,k,2)

              ! Slope of phi along y direction
              dphin   = phiold(i,j+1,k)-phiold(i,j-1,k)
              dphinp1 = phi   (i,j+1,k)-phi   (i,j-1,k)
              
              rhou(i,j,k,2) = rhou(i,j,k,2) - half*(rhon *dphin+rhonp1 *dphinp1)*dtdy/xc
              E   (i,j,k  ) = E   (i,j,k  ) - half*(rhoun*dphin+rhounp1*dphinp1)*dtdy/xc
              
           end do
        end do
     end do
  endif

  ! Gravitational acceleration along third direction
  if (ndim == 3) then
     do k = 1,nx(3)
        do j = 1,nx(2)
           do i = 1,nx(1)
              
              zkm1 = half*(x(k-1,3)+x(k  ,3))
              zkp1 = half*(x(k+1,3)+x(k+2,3))
              dtdz = dt/(zkp1-zkm1)
              xc = 1. ; sinxc = 1.
              if (Spherique) then
                 xc    = half*(x(i,1)+x(i+1,1))
                 sinxc = sin(half*(x(j,2)+x(j+1,2)))
              endif
              
              ! density
              rhonp1 = rho   (i,j,k)
              rhon   = rhoold(i,j,k)

              ! momentum
              rhoun   = rhouold(i,j,k,3)
              rhounp1 = rhou   (i,j,k,3)

              ! Slope of phi along z direction
              dphin   = phiold(i,j,k+1)-phiold(i,j,k-1)
              dphinp1 = phi   (i,j,k+1)-phi   (i,j,k-1)
              
              rhou(i,j,k,3) = rhou(i,j,k,3) - half*(rhon *dphin+rhonp1 *dphinp1)*dtdz/xc/sinxc
              E   (i,j,k  ) = E   (i,j,k  ) - half*(rhoun*dphin+rhounp1*dphinp1)*dtdz/xc/sinxc
              
           end do
        end do
     end do
  endif

  return
  
end subroutine update_gravity
