!> \file
!! Contains subroutine rd_restart_h5()
!<

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Subroutine RD_RESTART_H5
!
!> Reads a hdf5 output file to perform a simulation
!! restart.
!<
subroutine rd_restart_h5(k)

#if HDF==1
  use hdf5
  use parameters
  use var
  use varold
  use varray
  use divers
  use divers_ray, only : ngrp_ray
  use unites
  use unites_sortie
  use gammas
  use geom     , only : x_glob,x,shift_gr,nshift_gr,geometrie
  use para     , only : mype,x_start,x_end
  use cellules , only : ioffset, ifich,Pcell,N_cell
  use diffusion, only : erd => Er_dif,ngrp_dif
  use varray   ,        er => Eray,fr => Fray
  use rdwrt_h5

  implicit none
  
  integer, parameter         :: nb_int=23,nb_real=5
  integer, dimension(nb_int) :: para_int
  integer, dimension(3     ) :: nx_p
  integer                    :: nl1,nl2,nl3
  integer                    :: i,j,k,l,NNx,NNy,NNz,ish
  integer(HID_T)             :: file_id                   !< File identifiers
  integer                    :: error                     !< Flag to check operation success
  integer                    :: n1,n2,n3,n4
  
  double precision, dimension(nb_real) :: para_real
  
  real, dimension(:,:,:), allocatable :: T1
  real                                :: U,P,T

  character(len=23) :: filename 
  character(len=5 ) :: para_string 

  do i = 1,3
     nx_p(i) = nx(i)
  enddo

  !call get_filename(filename,'-r-000_000.h5',trepr(irepr),irepr,mype)
  !call get_filename(filename,'-00000_000.hdf',trepr(irepr),irepr,mype)
  filename = 'XXX/myjob-00000_000.hdf'  
  call make_name(filename,mype,i_restart)
  if (mype==0) write(*,*) 'Reading file :',filename

  !
  ! Initialize FORTRAN interface. 
  !
  call H5open_f(error) 
  call H5Fopen_f(trim(filename),H5F_ACC_RDONLY_F,file_id,error)

  ! array para_int ; Type integer; dim 1; size 19
  call get_1d_array_int_h5(file_id,"para_int",para_int,nb_int)
  nx_glob(1) = para_int(1)	  
  nx_glob(2) = para_int(2)	
  nx_glob(3) = para_int(3)	    
  ndim       = para_int(4)	  
  nx(1)      = para_int(5)	  
  nx(2)      = para_int(6)	
  nx(3)      = para_int(7)
  nfx        = para_int(8)
  x_start(1) = para_int(9)	  
  x_start(2) = para_int(10) 
  x_start(3) = para_int(11)	  
  x_end(1)   = para_int(12)	
  x_end(2)   = para_int(13)	    
  x_end(3)   = para_int(14)	  
  k          = para_int(15)

  if(para_int(16)==0 .and. RAY) then 
     write(*,*) " Previous job run without RAY option "
     call close_program(mype,'COULD NOT RESTART JOB WITH RAY OPTION')
  endif
  if(para_int(16)==1) write(*,*) " Previous job run with RAY option "

  if(para_int(17)==0 .and. DIFFU) then 
     write(*,*) " Previous job run without DIFFU option "
     call close_program(mype,'COULD NOT RESTART JOB WITH DIFFU OPTION')
  endif
  if(para_int(17)==1) write(*,*) " Previous job run with DIFFU option "

  if(para_int(18)==0 .and. CONDUC) then
     write(*,*) " Previous job run without CONDUC option "
     call close_program(mype,'COULD NOT RESTART JOB WITH CONDUC OPTION')
  endif
  if(para_int(18)==1) write(*,*) " Previous job run with CONDUC option "

#if MHD==1
  if(para_int(19)==0) then
     write(*,*) " Previous job run without MHD option "
     call close_program(mype,'COULD NOT RESTART JOB WITH MHD OPTION')
  endif
#endif

  if(para_int(20)==0 .and. GRAV) then
     write(*,*) " Previous job run without GRAV option "
     call close_program(mype,'COULD NOT RESTART JOB WITH GRAV OPTION')
  endif
  if(para_int(20)==1) write(*,*) " Previous job run with GRAV option "

  if(para_int(21).ne.nbuf) then
     write(*,*) " Previous job run with Nbuf= ",para_int(21)
     call close_program(mype,'COULD NOT RESTART JOB WITH DIFFERENT NBUF')
  endif

  if(para_int(22).ne.ngrp_ray) then
     write(*,*) " Previous job run with ngrp_ray= ",para_int(22)
     call close_program(mype,'COULD NOT RESTART JOB WITH DIFFERENT NGRP_RAY')
  endif

  if(para_int(23).ne.ngrp_dif) then
     write(*,*) " Previous job run with ngrp_dif= ",para_int(23)
     call close_program(mype,'COULD NOT RESTART JOB WITH DIFFERENT NGRP_DIF')
  endif

  do i = 1,3
     if (nx(i) .ne. nx_p(i)) then
        write(*,*) 'Pb. in the restart file'
        write(*,*) 'idim  =',i
        write(*,*) ' nx   =', nx  (i)
        write(*,*) 'nx_p  =', nx_p(i)
        call close_program(mype,'nx(i) NOT EQ nx_p(i)')
     endif
  enddo

  !Array sizes
  call get_nl(ndim,nl1,nl2,nl3)
  n1 = nx(1)+2*Nbuf*nl1 ; n2 = nx(2)+2*Nbuf*nl2 ; n3 = nx(3)+2*Nbuf*nl3

  !String geometry, length=5, dim=1
  call get_1d_array_string_h5(file_id,"geometry",para_string,5)
  if(para_string.ne.geometrie) then
     write(*,*) " Previous job run with geometry= ",para_string
     call close_program(mype,'COULD NOT RESTART JOB WITH DIFFERENT GEOMETRY')
  endif

  ! array para_real ; type real; dim 1; size 8
  call get_1d_array_h5(file_id,"para_real",para_real,nb_real)
  temps = para_real(1)
  dt    = para_real(2)
  dtold = para_real(3)
  gamma = para_real(4)
  mu    = para_real(5)
  u_M   = para_real(6)
  u_L   = para_real(7)
  u_T   = para_real(8)

  ! array x_glob ; type real; dim 1;   size nx_glob(1)+1;
  call get_1d_array_h5(file_id,"x_glob",x_glob(1:nx_glob(1)+1,1),nx_glob(1)+1)

  !Array "y_glob"; Type real; dim=1, size=nx_glob(2)+1
  call get_1d_array_h5(file_id,"y_glob",x_glob(1:nx_glob(2)+1,2)/u_L,nx_glob(2)+1)

  !Array "z_glob"; Type real; dim=1, size=nx_glob(3)+1
  call get_1d_array_h5(file_id,"z_glob",x_glob(1:nx_glob(2)+1,3)/u_L,nx_glob(3)+1)

  !Array "x"; Type real; dim=1, size=nx(1)+1
  call get_1d_array_h5(file_id,"x",x(1:nx(1)+1,1)/u_L,nx(1)+1)

  !Array "y"; Type real; dim=1, size=nx(2)+1
  call get_1d_array_h5(file_id,"y",x(1:nx(2)+1,2)/u_L,nx(2)+1)

  !Array "z"; Type real; dim=1, size=nx(3)+1
  call get_1d_array_h5(file_id,"z",x(1:nx(3)+1,3)/u_L,nx(3)+1)

  !Array "shift_gr"; Type real; dim=1, size=ndim
  call get_1d_array_h5(file_id,"shift_gr",shift_gr(1:ndim),ndim)

  !Array "nshift_gr"; Type integer; dim=1, size=ndim
  call get_1d_array_int_h5(file_id,"nshift_gr",nshift_gr(1:ndim),ndim)

  !Array "rho"; Type real; dim=3; size=(n1,n2,n3)
  call get_3d_array_h5(file_id,"rho",rho,n1,n2,n3)

  !Array "rhou"; Type real; dim=4; size=(n1,n2,n3,n4)
  n4 = N_vit
  call get_4d_array_h5(file_id,"rhou",rhou,n1,n2,n3,n4)

  !Array "E"; Type real; dim=3; size=(n1,n2,n3)
  call get_3d_array_h5(file_id,"E",E,n1,n2,n3)

#if SCALP==1
  if (nfx>=1) then
     n4 = nfx
     !Array "fx"; Type real; dim=3; size=(n1,n2,n3)
     call get_4d_array_h5(file_id,"fx",fx,n1,n2,n3,n4)
  endif
#endif

  if (RAY) then
     !Array "Eray"; Type real; dim=3, size=(n1,n2,n3)
     call get_3d_array_h5(file_id,"er",er,n1,n2,n3)
     !Array "Fray"; Type real; dim=4, size=(n1,n2,n3,n4) 
     n4 = ndim
     call get_4d_array_h5(file_id,"fr",fr,n1,n2,n3,n4)
  end if

  !Array "Er_dir"; Type real; dim=3, size=(n1,n2,n3)
  if (DIFFU) then
     call get_3d_array_h5(file_id,"erd",erd,n1,n2,n3)
  end if

#if MHD==1
  !Array "B"; Type real; dim=4; size=(n1,n2,n3)
  n4=N_vit
  call get_4d_array_h5(file_id,"B",B,n1,n2,n3,n4)
#endif

  !
  ! Close  file
  ! 
  call h5fclose_f(file_id, error)
  !
  ! Close FORTRAN interface. 
  !
  call h5close_f(error) 

  ! Go back to code units
  u_M = u_M * gramme
  u_L = u_L * centimetre
  u_T = u_T * seconde

  u_Vol  = u_L**3
  u_dens = u_M/u_Vol
  u_Vit  = u_L/u_T
  u_E    = u_M * u_Vit**2
  u_Mom  = u_dens * u_Vit
  u_Evol = u_E/u_Vol
  u_Fr   = u_vit * u_Evol
  
  x      = x      * u_L
  x_glob = x_glob * u_L
  rho    = rho    * u_dens
  rhou   = rhou   * u_Mom
  E      = E      * u_Evol
  if (RAY) then
     er = er * u_Evol
     fr = fr * u_Fr
  endif
  if (DIFFU) then
     erd = erd * u_Evol
  endif

  temps = temps*u_T ; dt = dt*u_T ; dtold = dtold*u_T
  
  return

#else

  use para, only : mype

  implicit none

  integer :: k
  
  call close_program(mype,'hdf5 dummy: rd_restart_h5')
  
#endif

end subroutine rd_restart_h5
