!> \file
!! Contains subroutine wrt_main_h5()
!<

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Subroutine WRT_MAIN_H5
!
!> Create_main_hdf5_file create "myjob000_main_hdf5_file"
!!
!! This program creates a the main file of the job containing 11 datasets 
!! which are:
!!    1   character*20 name_of_job
!!    2   character*80 comments
!!    3   integer, dimension(7) job_params
!!    4   character*80,dimension(ntotal)   datanames 
!!    5   character*80,dimension(ntotal)   dataunits
!!    6   integer , dimension(ntotal)    fourdim
!!    7   data_int list of integer parameters
!!    8   data_real list of real parameters
!!    9   xglob array
!!    10  yglob array
!!    11  zglob array
!!    12  shift_gr array
!!    13  nshift_gr array
!<
subroutine wrt_main_h5(k)

#if HDF==1
  use hdf5
  use rdwrt_h5
  use parameters
  use var
  use varold
  use varray
  use divers
  use divers_ray   , only : ngrp_ray
  use diffusion    , only : ngrp_dif
  use unites
  use unites_sortie
  use gammas
  use geom         , only : x_glob,x,shift_gr,nshift_gr,geometrie
  use para         , only : mype,ncpu,x_start,x_end
  use cellules     , only : ioffset,ifich,Pcell,N_cell

  implicit none

  integer         , dimension(23) :: para_int
  integer         , dimension(7 ) :: job_params
  integer(HSIZE_T), dimension(1 ) :: dims
  integer(  HID_T), dimension(9 ) :: dt_id
  integer(  HID_T), dimension(2 ) :: dtid
  integer(  HID_T)                :: file_id               !< File identifier 
  integer(  HID_T)                :: dset_id               !< Dataset identifier 
  integer(  HID_T)                :: dspace_id             !< Dataspace identifier
  integer(  HID_T)                :: dtype_id              !< Compound datatype identifier
  integer(  HID_T)                :: dtnid
  integer(  HID_T)                :: plist_id              !< Dataset trasfer property
  integer( SIZE_T), dimension(2 ) :: type_sizec    
  integer( SIZE_T)                :: type_sizei            !< Size of the integer datatype
  integer( SIZE_T)                :: offset
  integer( SIZE_T)                :: typesize,typesize1
  integer                         :: error                 !< Error flag
  integer                         :: i,j,l,k
  integer                         :: nproc,ntotal
  
  real, dimension(8 ) :: para_real

  character(len=8 ), parameter :: dsetname = "Compound"    !< Dataset name
  character(len=32)            :: filename 

  nproc = ncpu
  ntotal = ndata + nsupp

  filename = trim(name_of_job)//'.h5'
  write(*,*) 'Writing in file: ',filename
  write(*,*) 'Jobs uses: ',nproc, ' cpus.'

  !
  ! Initialize FORTRAN interface.
  !
  call h5open_f(error)
  call h5fcreate_f(filename, H5F_ACC_TRUNC_F, file_id, error)

  ! array name_of_job ; contains characters*20; dim 1; size ntotal;
  dims=(/ 1 /)
  call h5tcopy_f(H5T_NATIVE_CHARACTER,dtnid,error)
  typesize=80
  call h5tset_size_f(dtnid,typesize,error)
  call h5tget_size_f(dtnid,typesize1,error)
  call h5screate_simple_f(1,dims,dspace_id,error)
  call h5dcreate_f(file_id,"name_of_job",dtnid,dspace_id,dset_id,error)
  call h5dwrite_f(dset_id,dtnid,name_of_job,dims,error)
  call h5tclose_f(dtnid,error)
  call h5dclose_f(dset_id,error)
  call h5sclose_f(dspace_id,error)

  ! array comments ; contains characters*80; dim 1; size ntotal;
  dims=(/ 1 /)
  call h5tcopy_f(H5T_NATIVE_CHARACTER,dtnid,error)
  typesize=80
  call h5tset_size_f(dtnid,typesize,error)
  call h5tget_size_f(dtnid,typesize1,error)
  call h5screate_simple_f(1,dims,dspace_id,error)
  call h5dcreate_f(file_id,"comments",dtnid,dspace_id,dset_id,error)
  call h5dwrite_f(dset_id,dtnid,comments,dims,error)
  call h5tclose_f(dtnid,error)
  call h5dclose_f(dset_id,error)
  call h5sclose_f(dspace_id,error)

  ! Array "job_params"; Type integer; dim=1; size=7
  job_params=(/ nproc,iout-1,noutd,irepr-1,nreprd,ntotal,nbuf /)
  call dump_1d_array_int_h5(file_id,"job_params",job_params,7)

  ! array datanames ; contains characters*80 parameters
  ! real; dim 1;   size ntotal;
  do i=1,nsupp
     datanames(ndata+i) = suppnames(i)
  enddo
  dims=(/ntotal/)
  call h5tcopy_f(H5T_NATIVE_CHARACTER,dtnid,error)
  typesize=80
  call h5tset_size_f(dtnid,typesize,error)
  call h5tget_size_f(dtnid,typesize1,error)
  call h5screate_simple_f(1, dims,dspace_id,error)
  call h5dcreate_f(file_id,"datanames",dtnid,dspace_id,dset_id, error)
  call h5dwrite_f(dset_id, dtnid, datanames(1:ntotal), dims, error)
  call h5tclose_f(dtnid, error)
  call h5dclose_f(dset_id, error)
  call h5sclose_f(dspace_id, error)

  ! array dataunits ; contains characters*80 parameters
  ! real; dim 1;   size ntotal;
  do i=1,nsupp
     dataunits(ndata+i) = suppunits(i)
  enddo

  dims=(/ntotal/)
  call h5tcopy_f(H5T_NATIVE_CHARACTER,dtnid,error)
  typesize=80
  call h5tset_size_f(dtnid,typesize,error)
  call h5tget_size_f(dtnid,typesize1,error)
  call h5screate_simple_f(1,dims,dspace_id,error)
  call h5dcreate_f(file_id,"dataunits",dtnid,dspace_id,dset_id,error)
  call h5dwrite_f(dset_id, dtnid, dataunits(1:ntotal), dims, error)
  call h5tclose_f(dtnid, error)
  call h5dclose_f(dset_id, error)
  call h5sclose_f(dspace_id, error)

  !Array "fourdim"; Type integer; dim=1, size=ntotal
  do i=1,ndata
     if(datanames(i)=="rho" ) fourdim(i)=1
     if(datanames(i)=="rhou") fourdim(i)=N_vit      
     if(datanames(i)=="E"   ) fourdim(i)=1
     if(datanames(i)=="fx"  ) fourdim(i)=nfx
     if(datanames(i)=="er"  ) fourdim(i)=1
     if(datanames(i)=="fr"  ) fourdim(i)=ndim
     if(datanames(i)=="erd" ) fourdim(i)=1
     if(datanames(i)=="B"   ) fourdim(i)=N_vit
  end do
  do i=1,nsupp
     fourdim(ndata+i)= suppdim(i)
  enddo
  call dump_1d_array_int_h5(file_id,"fourdim",fourdim(1:ntotal),ntotal)

  ! Array "para_int"; Type integer; dim=1; size=23
  para_int( 1) = nx_glob(1)	  
  para_int( 2) = nx_glob(2)	
  para_int( 3) = nx_glob(3)	    
  para_int( 4) = ndim	  
  para_int( 5) = nx(1)	  
  para_int( 6) = nx(2)	
  para_int( 7) = nx(3)
  para_int( 8) = nfx
  para_int( 9) = x_start(1)	  
  para_int(10) = x_start(2) 
  para_int(11) = x_start(3)	  
  para_int(12) = x_end(1)	
  para_int(13) = x_end(2)	    
  para_int(14) = x_end(3)	  
  para_int(15) = k	  
  para_int(16) = 0
  if(RAY) para_int(16) = 1	
  para_int(17) = 0
  if(DIFFU) para_int(17) = 1	
  para_int(18) = 0
  if(CONDUC) para_int(18) = 1	
#if MHD==1
  para_int(19) = 1
#else
  para_int(19) = 0
#endif
  para_int(20) = 0
  if(GRAV) para_int(20) = 1
  para_int(21) = Nbuf
  para_int(22) = Ngrp_ray
  para_int(23) = Ngrp_dif

  call dump_1d_array_int_h5(file_id,"para_int",para_int,23)

  !String geometry, length=5, dim=1
  call dump_1d_array_string_h5(file_id,"geometry",geometrie,5)
 
  !Array "para_real"; Type real; dim=1; size=8
  para_real= (/ temps/u_T,dt/u_T,dtold/u_T,gamma,mu,u_M/gramme,u_L/centimetre,u_T/seconde /)
  call dump_1d_array_h5(file_id,"para_real",para_real,8)

  !Array "x_glob"; Type real; dim=1, size=nx_glob(1)+1
  call dump_1d_array_h5(file_id,"x_glob",x_glob(1:nx_glob(1)+1,1)/u_L,nx_glob(1)+1)

  !Array "y_glob"; Type real; dim=1, size=nx_glob(2)+1
  call dump_1d_array_h5(file_id,"y_glob",x_glob(1:nx_glob(2)+1,2)/u_L,nx_glob(2)+1)

  !Array "z_glob"; Type real; dim=1, size=nx_glob(3)+1
  call dump_1d_array_h5(file_id,"z_glob",x_glob(1:nx_glob(2)+1,3)/u_L,nx_glob(3)+1)

  !Array "shift_gr"; Type real; dim=1, size=ndim
  call dump_1d_array_h5(file_id,"shift_gr",shift_gr(1:ndim),ndim)

  !Array "nshift_gr"; Type integer; dim=1, size=ndim
  call dump_1d_array_int_h5(file_id,"nshift_gr",nshift_gr(1:ndim),ndim)

  ! 
  ! Close the file.
  !
  call h5fclose_f(file_id, error)

  ! Close FORTRAN interface.
  !
  call h5close_f(error)

  return

#else

  use para, only : mype

  implicit none

  integer :: k

  call close_program(mype,'hdf5 dummy: wrt_main_h5')

#endif

end subroutine wrt_main_h5
