!> \file
!! Contains subroutine hydro_step()
!<

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Subroutine HYDRO_STEP
!
!> Determines the size of the block which will be treated
!! in the evol_hydro and evol_mhd steps.
!<
subroutine hydro_step(t_hydro,temps_comm,monit)

  use parameters, only : nx
  use divers    , only : nsx,nsy,nsz,hydro,verbose
  use para      , only : mype
  
  implicit none

#if PARA==1
  include 'mpif.h'
#else
  real     :: MPI_Wtime
  external :: MPI_Wtime
#endif

  integer           :: i,j,k,ierr
  integer           :: ixm,ixp,iym,iyp,izm,izp
  real              :: t_hydro,tr1,tr2,temps_comm
  character(len=50) :: Com_Moni
  logical           :: monit

  if(verbose) write(*,*) 'Start of hydro_step'

#if MHD==1
  if(mype==0) write(*,'(a)') '************************* Magneto-hydrodynamics **************************'

  Com_Moni = 'Before MHD step...'
  if (monit) call monitoring(Com_Moni)

  tr1 = MPI_Wtime(ierr)     
  do i=1,nsx
     do j=1,nsy
        do k=1,nsz
           ixm = 1 + (i-1)*nx(1)/nsx
           iym = 1 + (j-1)*nx(2)/nsy
           izm = 1 + (k-1)*nx(3)/nsz
           ixp = i*nx(1)/nsx
           iyp = j*nx(2)/nsy
           izp = k*nx(3)/nsz
           call evol_mhd(ixm,ixp,iym,iyp,izm,izp)
        enddo
     enddo
  enddo
  tr2 = MPI_Wtime(ierr)     
  t_hydro = t_hydro + tr2-tr1

  Com_Moni = 'After MHD step...'
  if (monit) call monitoring(Com_Moni)
  if(mype==0) write(*,'(a)') '**************************************************************************'

#else

  if (hydro) then
  
     if(mype==0) write(*,'(a)') '***************************** Hydrodynamics ******************************'

     Com_Moni = 'Before HYDRO step...'
     if (monit) call monitoring(Com_Moni)
     
     tr1 = MPI_Wtime(ierr)     
     do i=1,nsx
        do j=1,nsy
           do k=1,nsz
              ixm = 1 + (i-1)*nx(1)/nsx
              iym = 1 + (j-1)*nx(2)/nsy
              izm = 1 + (k-1)*nx(3)/nsz
              ixp = i*nx(1)/nsx
              iyp = j*nx(2)/nsy
              izp = k*nx(3)/nsz    
              call evol_hydro(ixm,ixp,iym,iyp,izm,izp)
           enddo
        enddo
     enddo
     tr2 = MPI_Wtime(ierr)     
     t_hydro = t_hydro + tr2-tr1    
     
     Com_Moni = 'After HYDRO step...'
     if (monit) call monitoring(Com_Moni)

     if(mype==0) write(*,'(a)') '**************************************************************************'

  endif

#endif

  call communications(temps_comm)
  call limites

  if(verbose) write(*,*) 'End of hydro_step'

  return
  
end subroutine hydro_step
