!> \file
!! Contains subroutine monitoring()
!<

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Subroutine MONITORING
!
!> Monitors the minimum, maximum and average values of the
!! gas density, pressure, sound speed, Mach number and
!! temperature in the entire grid. The status of each
!! variable is printed to screen.
!<
subroutine monitoring(Comment)

  use parameters
  use var
  use varold
  use varray
  use divers
  use divers_ray, only : dt_ray,dt_ray_exp,imp_ray,exp_ray
  use unites
  use para

  implicit none

#if PARA==1
  include 'mpif.h'
  Integer           :: info 
  real              :: xx
#endif

  integer,dimension(3) :: ii
  
  real, dimension(:,:,:),allocatable :: rr
  real :: Mach_min,Mach_max, Pmin,Pmax,Rhomin,rhomax, csmin,csmax, Tmin, Tmax
  real :: Pmoy,rhomoy, Mach_moy, csmoy, Tmoy, Vol

  character(len=50) :: Comment
  

  if (mype==0) write(*,*) Comment

  allocate(rr(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)))

  Mach_min =  1.0E50 ; Pmin =  1.0E50 ; Rhomin =  1.0E50 ; csmin =  1.0E50 ; Tmin =  1.0E50
  Mach_max = -1.0E50 ; Pmax = -1.0E50 ; rhomax = -1.0E50 ; csmax = -1.0E50 ; Tmax = -1.0E50

  rhomin = minval(rho(1:nx(1),1:nx(2),1:nx(3)))
  rhomax = maxval(rho(1:nx(1),1:nx(2),1:nx(3)))
  rhomoy =    sum(rho(1:nx(1),1:nx(2),1:nx(3)))

  ! compute Ein
  E = E - 0.5*sum(rhou**2,4)/rho
  if (allocated(B)) E = E - 0.5*sum(B**2,4)

  call pressure(rho,E,rr,nxmin(1),nxmax(1),nxmin(2),nxmax(2),nxmin(3),nxmax(3))

  Pmin = minval(rr(1:nx(1),1:nx(2),1:nx(3)))
  Pmax = maxval(rr(1:nx(1),1:nx(2),1:nx(3)))
  Pmoy =    sum(rr(1:nx(1),1:nx(2),1:nx(3)))

  if (Pmin < 0.) then
     ii=minloc(rr)
     write(*,*) 'i,j,k :',nxmin(1)-1+ii(1),nxmin(2)-1+ii(2),nxmin(3)-1+ii(3)
     write(*,*) 'P =', rr(nxmin(1)-1+ii(1),nxmin(2)-1+ii(2),nxmin(3)-1+ii(3))
     write(*,*) 'E =',  E(nxmin(1)-1+ii(1),nxmin(2)-1+ii(2),nxmin(3)-1+ii(3))
     call close_program(mype,'called from MONITORING: Pmin < 0')
  endif

  call temperature(rho,E,rr,nxmin(1),nxmax(1),nxmin(2),nxmax(2),nxmin(3),nxmax(3))
  Tmin = minval(rr(1:nx(1),1:nx(2),1:nx(3)))
  Tmax = maxval(rr(1:nx(1),1:nx(2),1:nx(3)))
  Tmoy =    sum(rr(1:nx(1),1:nx(2),1:nx(3)))

  call sound_speed(rho,E,rr,nxmin(1),nxmax(1),nxmin(2),nxmax(2),nxmin(3),nxmax(3))
  csmin = minval(rr(1:nx(1),1:nx(2),1:nx(3)))
  csmax = maxval(rr(1:nx(1),1:nx(2),1:nx(3)))
  csmoy =    sum(rr(1:nx(1),1:nx(2),1:nx(3)))
  
  rr = sqrt(sum(rhou**2,4)/rho**2)/rr
  Mach_min = minval(rr(1:nx(1),1:nx(2),1:nx(3)))
  Mach_max = maxval(rr(1:nx(1),1:nx(2),1:nx(3)))
  Mach_moy =    sum(rr(1:nx(1),1:nx(2),1:nx(3)))

  ! restore E 
  E = E + 0.5*sum(rhou**2,4)/rho
  if (allocated(B)) E = E + 0.5*sum(B**2,4)

#if PARA==1
  call MPI_ALLREDUCE(Pmin    , xx ,1,MPI_DOUBLE_PRECISION,MPI_MIN,MPI_COMM_WORLD,info) ; Pmin = xx
  call MPI_ALLREDUCE(Tmin    , xx ,1,MPI_DOUBLE_PRECISION,MPI_MIN,MPI_COMM_WORLD,info) ; Tmin = xx
  call MPI_ALLREDUCE(rhomin  , xx ,1,MPI_DOUBLE_PRECISION,MPI_MIN,MPI_COMM_WORLD,info) ; Rhomin = xx
  call MPI_ALLREDUCE(csmin   , xx ,1,MPI_DOUBLE_PRECISION,MPI_MIN,MPI_COMM_WORLD,info) ; csmin = xx
  call MPI_ALLREDUCE(Mach_min, xx ,1,MPI_DOUBLE_PRECISION,MPI_MIN,MPI_COMM_WORLD,info) ; Mach_min = xx
  
  call MPI_ALLREDUCE(Pmax    , xx ,1,MPI_DOUBLE_PRECISION,MPI_MAX,MPI_COMM_WORLD,info) ; Pmax = xx
  call MPI_ALLREDUCE(Tmax    , xx ,1,MPI_DOUBLE_PRECISION,MPI_MAX,MPI_COMM_WORLD,info) ; Tmax = xx
  call MPI_ALLREDUCE(rhomax  , xx ,1,MPI_DOUBLE_PRECISION,MPI_MAX,MPI_COMM_WORLD,info) ; rhomax = xx
  call MPI_ALLREDUCE(csmax   , xx ,1,MPI_DOUBLE_PRECISION,MPI_MAX,MPI_COMM_WORLD,info) ; csmax = xx
  call MPI_ALLREDUCE(Mach_max, xx ,1,MPI_DOUBLE_PRECISION,MPI_MAX,MPI_COMM_WORLD,info) ; mach_max = xx

  call MPI_ALLREDUCE(Pmoy    , xx ,1,MPI_DOUBLE_PRECISION,MPI_SUM,MPI_COMM_WORLD,info) ; Pmoy = xx
  call MPI_ALLREDUCE(Tmoy    , xx ,1,MPI_DOUBLE_PRECISION,MPI_SUM,MPI_COMM_WORLD,info) ; Tmoy = xx
  call MPI_ALLREDUCE(rhomoy  , xx ,1,MPI_DOUBLE_PRECISION,MPI_SUM,MPI_COMM_WORLD,info) ; rhomoy = xx
  call MPI_ALLREDUCE(csmoy   , xx ,1,MPI_DOUBLE_PRECISION,MPI_SUM,MPI_COMM_WORLD,info) ; csmoy = xx
  call MPI_ALLREDUCE(Mach_moy, xx ,1,MPI_DOUBLE_PRECISION,MPI_SUM,MPI_COMM_WORLD,info) ; mach_moy = xx
#endif

  Vol = float(nx_glob(1))*float(nx_glob(2))*float(nx_glob(3))
  Pmoy     = Pmoy     / Vol
  rhomoy   = rhomoy   / Vol
  Mach_moy = Mach_moy / Vol
  csmoy    = Csmoy    / Vol
  Tmoy     = Tmoy     / Vol

  if (mype == 0) then
     write(*,401) rhomin,rhomax, rhomoy
     write(*,402) Pmin,Pmax, Pmoy
     write(*,403) csmin,csmax, csmoy
     write(*,404) Mach_min,mach_max, mach_moy
     write(*,405) Tmin,Tmax,Tmoy
  endif

401 format('Rhomin, Rhomax, Rhomoy =',1pe12.3,2(2x,E12.3))
402 format('Pmin  , Pmax  , Pmoy   =',1pe12.3,2(2x,E12.3))
403 format('Csmin , Csmax , Csmoy  =',1pe12.3,2(2x,E12.3))
404 format('Mmin  , Mmax  , Mmoy   =',1pe12.3,2(2x,E12.3))
405 format('Tmin  , Tmax  , Tmoy   =',1pe12.3,2(2x,E12.3))

  deallocate(rr)

  return

end subroutine monitoring
