!> \file
!! Contains subroutine rh() and function f()
!<

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Subroutine RH
!
!> Computes Rankine-Hugoniot jump conditions.
!<
subroutine rh(rho1,u1,T1,rho2,u2,T2)

  use unites
  use gammas
  use divers, only : mu,RAY
  use para  , only : mype
  
  implicit none

  real :: u1,rho1,pg1,pr1,T1
  real :: u2,rho2,T2
  real :: M1,PP,alpha1,r,cc
  real :: eps,eps_r,equ1,equ2
  real :: PP0,r0,PPlow,PPhigh,f,f1,f2,xx,M1b,M1c,rlow,rhigh

!  real :: j11,j12,j21,j22,det,dPP,dr,fl

  u1 = abs(u1)
  write(*,998) rho1/(gramme/cm3),u1/kms,T1/degre
998 format( 'rho1,u1,T1 =',1pe10.4,3(2x,E10.4))

  pg1 = rho1 *kb*T1/(mu*uma)
  pr1 = a_r*T1**4 /3.

  write(*,*) 'Pgaz1 =', pg1/Pascal
  M1     = abs(u1)/sqrt(gamma*pg1/rho1) 
  alpha1 = pr1/pg1
 
  r0 = (gamma+1.)/(gamma-1.)

  write(*,900) M1 
900 format('M1        = ',1pe10.4)
  write(*,901) alpha1 
901 format('alpha1    = ',1pe10.4)

  r  = (gamma+1)*M1**2/((gamma-1.)*M1**2+2.)
  PP = (2.*gamma*M1**2-(gamma-1.))/(gamma+1.)
  T2   = pg1*PP*(mu*uma)/(rho1*r)/kb
  rho2 = rho1*r
  u2   = u1 / r

  write(*,*)
  write(*,*) 'Values without Radiation:'
  write(*,201) r
  write(*,202) PP
  write(*,203) T2   /degre
  write(*,204) rho2 /(gramme/cm3)

  if(RAY) then
  rlow  = min(r,7.)
  rhigh = max(r,7.)

  write(*,*) 'rlow, rhigh =',rlow, rhigh
  eps_r = 1.

  do while (eps_r > 1.0E-14) 

     r  = 0.5*(rlow+rhigh)
     cc = r - 7.

     ! For fixed r (compression factor), find PP (the pressure ratio)
     xx = (r-r0)*r**4 / (4.*alpha1*(7.-r))
  
     if (xx < 1.) then
        PP0 = 1.
     else
        PP0 = (abs(xx))**(1./3.)
     endif

     PPlow = PP0
     f1 = f(PPlow,r,r0,alpha1)
     if (f1*cc < 0.) then
        write(*,*) 'problem f1 > 0.  f1=',f1
        call close_program(mype,'called from RH')
     endif
     f2 = f1
     PPhigh = PPlow
     do while( f2*f1 > 0.)
        PPhigh = PPhigh*2.
        f2 = f(PPhigh,r,r0,alpha1)
     enddo
  
     eps = 1.
     do while (eps > 1.0E-14)
        PP = 0.5*(PPlow + PPhigh)
        f1 = f(PP,r,r0,alpha1)
        if (f1*cc > 0.) then
           PPlow  = PP
        else 
           PPhigh = PP
        endif
        eps = 2.*(PPhigh - PPlow)/(PPhigh + PPlow)
     enddo

     M1b = sqrt( ( (PP-1.) + alpha1*((PP/r)**4 - 1.))*r/(r-1.)/gamma )
     M1c = sqrt((gamma/(gamma-1.)*(PP/r-1.) + 4.*alpha1*((PP/r)**4/r -1.))*r*r/(r*r-1.)*2./gamma)

     if (cc*M1b < cc*M1) then
        rhigh = r
     else
        rlow  = r
     endif

     !write(*,991) PP
     !write(*,992) M1,M1b,M1c
     !write(*,993) rlow, rhigh
     !read (*,*) r
     eps_r = rhigh-rlow


     !write(10,999) r,PP,M1b
     !read(*,*)
  enddo

!999 format(1pe10.4,5(2x,E10.4))
!991 format(' PP          =',1pe11.4,5(2x,E11.4))
!992 format(' M1          =',1pe11.4,5(2x,E11.4))
!993 format(' rlow, rhigh =',1pe11.4,5(2x,E11.4))

  equ1 = gamma*M1**2 *(r-1.)/r -(PP-1.) - alpha1 * (PP**4/r**4 - 1.) 
  equ2 = 0.5*gamma*M1**2*(r**2-1.)/r**2 - gamma/(gamma-1.) *(PP/r-1.) - 4.*alpha1*(PP**4/r**5 -1.)

  T2   = pg1*PP*(mu*uma)/(rho1*r)/kb
  rho2 = rho1*r
  u2   = u1 / r

  write(*,*)
  write(*,*) 'Values with radiation:'
  write(*,201) r
  write(*,202) PP
  write(*,203) T2  /degre
  write(*,204) rho2/(gramme/cm3)
  write(*,*)
  write(*,*) 'equ1,equ2 = ',equ1,equ2
  write(*,*)
  
201 format('r    = ',1pe10.4)
202 format('PP   = ',1pe10.4)
203 format('T2   = ',1pe10.4)
204 format('rho2 = ',1pe10.4)
 
  endif
  
  return

!  eps = 1.
!  r   = 4
!  PP  = 10.

!  do while (eps > 1.0E-12)

!     equ1 = gamma*M1**2 *(r-1.)/r -(PP-1.) - alpha1 * (PP**4/r**4 - 1.) 
!     equ2 = 0.5*gamma*M1**2*(r**2-1.)/r**2 - gamma/(gamma-1.) *(PP/r-1.) - 4.*alpha1*(PP**4/r**5 -1.)

!     j11  =  gamma*M1**2/r**2 + 4.*alpha1*(PP**4/r**5)
!     j12  = -1. - 4.*alpha1*(PP**3/r**4)
     
!     j21  =  gamma*M1**2/r**3 + gamma/(gamma-1.) *(PP/r**2) + 20.*alpha1*(PP**4/r**6)
!     j22  = - gamma/(gamma-1.)/r -16.*alpha1*PP**3/r**5


!     det  = j11*j22-j12*j21

!     dr   = -( j22*equ1 - j12*equ2)/det
!     dPP  = -(-j21*equ1 + j11*equ2)/det

!     fl = 1.
!     fl   = min(abs(r/dr),abs(PP/dPP))
!     fl   = min(1.,0.1*fl)
!     write(*,*) 'fl=',fl

!     r  = r  + fl*dr
!     PP = PP + fl*dPP

!     eps = max(abs(dr/r),abs(dPP/PP))

!     !write(*,*) 'r ,dr  = ',r ,fl*dr
!     !write(*,*) 'PP,dPP = ',PP,fl*dPP

!     !write(*,*)
!     !write(*,*) 'equ1,equ2 =', equ1,equ2
!     !write(*,*) 'eps=',eps
!     !read (*,*)
!  enddo

!  write(*,*) 'On a converge...'
!  write(*,*) 'r, PP =',r, PP
  
!  stop

end subroutine rh

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Function f
!
!> Find the zero of f to find PP (fixed r).
!<
function f(PP,r,r0,alpha1)

  implicit none

  real :: PP,r,r0,alpha1,f

  f = alpha1*(7.-r)*(PP/r)**4 - (r-r0)*PP -alpha1*(7.*r-1.) - (r0*r-1.)
  
  return

end function f
