!> \file
!! Contains subroutine allocate_array()
!<

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Subroutine ALLOCATE_ARRAY
!
!> Allocates all the main hydro and MHD variable arrays.
!<
subroutine allocate_array

  use parameters
  use var
  use varold
  use etat_gds
  use cl
  use divers
  use geom, only : x,x_glob,shift_gr,nshift_gr

  implicit none
  
  integer :: idim

  nxmin = 1
  nxmax = 1

  do idim=1,ndim
     nxmin(idim) =    1     - Nbuf
     nxmax(idim) = nx(idim) + Nbuf
  enddo

  ! allocate variables
  allocate( rho(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)        ))
  allocate(rhou(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3),1:N_vit))
  allocate(   E(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)        ))

  allocate( rhoold(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)        ))
  allocate(rhouold(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3),1:N_vit))
  allocate(   Eold(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)        ))


  ! allocate additional arrays according to nsupp and suppdim
  if(nsupp>=1) allocate( supp1(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3),1:suppdim(1)))
  if(nsupp>=2) allocate( supp2(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3),1:suppdim(2)))
  if(nsupp>=3) allocate( supp3(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3),1:suppdim(3)))
  if(nsupp>=4) allocate( supp4(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3),1:suppdim(4)))
  if(nsupp>=5) allocate( supp5(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3),1:suppdim(5)))
  if(nsupp> 5) write(*,*) 'Warning, only the 5 first additional arrays are taken into account'

  ! passive scalar arrays
#if SCALP==1
  allocate(  Fx   (nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3),1:nFx ))
  allocate(  Fxold(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3),1:nFx ))
  Fx = 0. ; Fxold = 0. 
  allocate(fx_l(1:nfx,1:2*ndim))
#endif

  ! grid arrays
  nx_glob_max = 1 ; nx_max = 1
  do idim=1,3
     nx_glob_max = max(nx_glob_max,nx_glob(idim))
     nx_max      = max(nx_max     ,nx(idim))
  enddo

  allocate(x     ((1-Nbuf):nx_max     +(Nbuf+1),3) )
  allocate(x_glob((1-Nbuf):nx_glob_max+(Nbuf+1),3))

   shift_gr =  0.
  nshift_gr =  0

 ! boundary condition arrays
  allocate(  rho_l(      1:2*ndim))
  allocate( rhou_l(ndim, 1:2*ndim))
  allocate(    u_l(ndim, 1:2*ndim))
  allocate(    E_l(      1:2*ndim))
  allocate(    P_l(      1:2*ndim))
  allocate(    T_l(      1:2*ndim))

  ! MHD arrays
#if MHD==1
  allocate(B   (nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3),1:N_vit))
  allocate(Bold(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3),1:N_vit))
  B = 0. ; Bold = 0.
  allocate(B_l(1:N_vit,1:2*ndim))
#endif

  rho = 0. ; rhou = 0. ; E = 0. ; rhoold = 0. ; rhouold = 0. ; Eold = 0. ; x_glob = 0. ; x = 0.
  
  return

end subroutine allocate_array
