!> \file
!! contains subroutine pasdt_mhd()
!<

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Subroutine PASDT_MHD
!
!> Computes the mhd timestep.
!<
subroutine pasdt_mhd

  use parameters
  use var    
  use gammas
  use unites    , only : an,seconde,kb,uma,kg,metre,degre
  use geom      , only : x,geom_dir
  use para      , only : mype
  use conduction
  use divers    , only : verbose,mu,dt_mhd
  use const
  
  implicit none

#if PARA==1
  include 'mpif.h'
  integer :: info
#endif

  ! local variables
  integer                  :: i,j,k,idim,i0,j0,k0,ixmin,ixmax,iymin,iymax,izmin,izmax
  real, dimension(1:ndim ) :: dx
  real, dimension(1:n_vit) :: Bc
  real                     :: dt, dt_glob
  real                     :: dens,vx,vy,vz,pression,bx,by,bz
  real                     :: af_x,af_y,af_z

  if (verbose) write(*,*) 'Start of pasdt_mhd'

  dt = 1.0E+50*an
  
  ixmin =  0 ; ixmax = nx(1)+1
  iymin =  1 ; iymax = 1
  izmin =  1 ; izmax = 1
  if(ndim.gt.1) then
     iymin =  0 ; iymax = nx(2)+1
  endif
  if(ndim.gt.2) then
     izmin =  0 ; izmax = nx(3)+1
  endif

  do k = izmin,izmax
  do j = iymin,iymax
  do i = ixmin,ixmax

     Bc(1:n_vit) = B(i,j,k,1:n_vit)
     do idim = 1,ndim
        i0 = 0 ; j0 = 0 ; k0 = 0
        if (idim==1) i0 = 1
        if (idim==2) j0 = 1
        if (idim==3) k0 = 1
        Bc(idim) = half*(B(i,j,k,idim)+B(i+i0,j+j0,k+k0,idim))
     enddo
     
     Pression = E(i,j,k)
     do idim = 1,N_vit
        Pression = Pression - 0.5*(rhou(i,j,k,idim)**2 / rho(i,j,k) +  Bc(idim)**2  )
     enddo
     Pression = (gamma-1.) * Pression

     dens = rho (i,j,k)
     vx   = rhou(i,j,k,1)/dens 
     vy   = 0.
     vz   = 0.
     bx   = Bc(1)
     by   = 0.
     bz   = 0.
     
     if (N_vit > 1 ) then
        vy = rhou(i,j,k,2)/dens
        by = Bc  (2)
     endif

     if (N_vit > 2) then
        vz = rhou(i,j,k,3)/dens
        bz = Bc  (3)
     endif

     !compute fast magnetosonic wave (x dir)
     af_x=0.d0 ; af_y=0d0 ; af_z=0.d0
     call fast_mhd_speed(dens,pression,vx,vy,vz,bx,by,bz,af_x)
     if (ndim > 1) then
        !compute fast magnetosonic wave (y dir)
        call fast_mhd_speed(dens,pression,vy,vx,vz,by,bx,bz,af_y)
     endif
     if (ndim > 2) then
        !compute fast magnetosonic wave (z dir)
        call fast_mhd_speed(dens,pression,vz,vx,vy,bz,bx,by,af_z)
     endif

     vx=abs(vx)+af_x
     vy=abs(vy)+af_y
     vz=abs(vz)+af_z

                      dx(1) = x(i+1,1)-x(i,1)
     if (ndim .gt. 1) dx(2) = x(j+1,2)-x(j,2)
     if (ndim .gt. 2) dx(3) = x(k+1,3)-x(k,3)

     if (geom_dir(2) == 'phi_c') dx(2) = 0.5*(x(i,1)+x(i+1,1))*dx(2)
     if (geom_dir(2) == 'the_s') dx(2) = 0.5*(x(i,1)+x(i+1,1))*dx(2)
     if (geom_dir(3) == 'phi_s') dx(3) = 0.5*(x(i,1)+x(i+1,1))*sin(0.5*(x(j,2)+x(j+1,2))) * dx(3)

     if (ndim == 1) then
        dt = min(dt, dx(1)/vx)
     else if (ndim == 2) then
        dt = min(dt, 1./(vx/dx(1) + vy/dx(2)))
     else if (ndim == 3) then
        dt = min (dt, 1./(vx/dx(1) + vy/dx(2) + vz/dx(3)))
     endif

  enddo
  enddo
  enddo

  ! Mhd timstep reduction
#if PARA==1
  info = 0
  call MPI_ALLREDUCE(dt, dt_glob,1,MPI_DOUBLE_PRECISION,MPI_MIN,MPI_COMM_WORLD,info)
#else
  dt_glob = dt
#endif
  
  dt_mhd = 0.8*dt_glob     

  if (verbose) write(*,*) 'Fin de pasdt_mhd'

  return
  
end subroutine pasdt_mhd
