!> \file
!! Contains subroutine diffusion_ex()
!<

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Subroutine DIFFUSION_EX
!
!> This routine solves the FLD equations explicitly.
!<
subroutine diffusion_ex(imin,imax,jmin,jmax,kmin,kmax)

  use var
  use var_loc
  use varold
  use geom
  use divers
  use parameters, only : ndim,nx,nxmin,nxmax
  use cl
  use para      , only : mype
  use unites    , only : c
  use diffusion , only : Er_dif,fld_limiter
  use cl_dif    , only : icl_dif

  implicit none

  integer, intent(in )    :: imin,imax,jmin,jmax,kmin,kmax

  integer, dimension(3,3) :: kron
  integer, dimension(6  ) :: icl_loc
  integer                 :: nnx,nny,nnz,nn
  integer                 :: i,j,k,idim,id
  integer                 :: ip1,jp1,kp1,im1,jm1,km1
  integer                 :: ixmin,ixmax,iymin,iymax,izmin,izmax

  real, dimension(:,:,:,:), allocatable :: k_gradE
  real, dimension(:,:,:  ), allocatable :: kappa_abs,kappa_sca,cv
  real, dimension(:,:,:  ), allocatable :: Er_difloc
  real                                  :: sig,flux_ener_tot,omega,den
  real                                  :: source_dif,derive_source_dif
  real                                  :: lambda_fld,xx,coeff

  if (verbose) write(*,*) 'Start of explicit diffusion'

  call check_size(imin,imax,jmin,jmax,kmin,kmax)

  ! Boundary conditions on the sub-arrays
  icl_loc = 0

  if(imin.eq.1    ) icl_loc(1)=icl(1)
  if(imax.eq.nx(1)) icl_loc(2)=icl(2)
  if(jmin.eq.1    ) icl_loc(3)=icl(3)
  if(jmax.eq.nx(2)) icl_loc(4)=icl(4)
  if(kmin.eq.1    ) icl_loc(5)=icl(5)
  if(kmax.eq.nx(3)) icl_loc(6)=icl(6)

  ! Arrays allocation
  nnx = imax - imin + 1 ; nny =  jmax - jmin + 1 ; nnz = kmax - kmin + 1
  if(ndim.eq.1) then
     ixmin =  0 ; ixmax = nnx+1
     iymin =  1 ; iymax = 1
     izmin =  1 ; izmax = 1
  else if (ndim == 2) then
     ixmin =  0 ; ixmax = nnx+1
     iymin =  0 ; iymax = nny+1
     izmin =  1 ; izmax = 1
  else if (ndim == 3) then
     ixmin =  0 ; ixmax = nnx+1
     iymin =  0 ; iymax = nny+1
     izmin =  0 ; izmax = nnz+1
  endif

  kron = 0
  do idim=1,3
     kron(idim,idim) = 1
  enddo

  nn  = max(nnx,nny,nnz)
  allocate(xloc(-1:nn+3,ndim)) 

  allocate( rholoc  (nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)       ))
  allocate(   Eloc  (nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)       ))
  allocate(   Tloc  (nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)       ))
  allocate(rhouloc  (nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3),1:ndim))
  allocate( k_gradE (nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3),1:ndim))
  allocate(kappa_abs(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)       ))
  allocate(kappa_sca(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)       ))
  allocate( cv      (nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)       ))
  allocate(Er_difloc(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)       ))

  if(verbose) write(*,*) 'Fill sub-arrays'

  ! Fill sub-arrays
  if (ndim == 1) then
        rholoc(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)       ) =  rhoold(imin-2:imax+2,1:1,1:1       )
          Eloc(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)       ) =    Eold(imin-2:imax+2,1:1,1:1       )
       rhouloc(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3),1:ndim) = rhouold(imin-2:imax+2,1:1,1:1,1:ndim)
     Er_difloc(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)       ) =  Er_dif(imin-2:imax+2,1:1,1:1       )
  else if (ndim ==2) then
        rholoc(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)       ) =  rhoold(imin-2:imax+2,jmin-2:jmax+2,1:1       )
          Eloc(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)       ) =    Eold(imin-2:imax+2,jmin-2:jmax+2,1:1       )
       rhouloc(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3),1:ndim) = rhouold(imin-2:imax+2,jmin-2:jmax+2,1:1,1:ndim)
     Er_difloc(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)       ) =  Er_dif(imin-2:imax+2,jmin-2:jmax+2,1:1       )
  else if (ndim ==3) then
        rholoc(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)       ) =  rhoold(imin-2:imax+2,jmin-2:jmax+2,kmin-2:kmax+2       )
          Eloc(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)       ) =    Eold(imin-2:imax+2,jmin-2:jmax+2,kmin-2:kmax+2       )
       rhouloc(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3),1:ndim) = rhouold(imin-2:imax+2,jmin-2:jmax+2,kmin-2:kmax+2,1:ndim)
     Er_difloc(nxmin(1):nxmax(1),nxmin(2):nxmax(2),nxmin(3):nxmax(3)       ) =  Er_dif(imin-2:imax+2,jmin-2:jmax+2,kmin-2:kmax+2       )
  endif

                 xloc(-1:nnx+3,1) = x(imin-2:imax+3,1)   
  if (ndim.gt.1) xloc(-1:nny+3,2) = x(jmin-2:jmax+3,2)   
  if (ndim.gt.2) xloc(-1:nnz+3,3) = x(kmin-2:kmax+3,3)   

  ! Compute geometry
  if(verbose) write(*,*) 'Compute geometry'
  call cal_geom(nnx,nny,nnz)

  ! Compute the temperature
  if(verbose) write(*,*) 'Compute Tloc'

  Eloc = Eloc - 0.5*sum(rhouloc**2,4)/rholoc
  if(allocated(B)) Eloc = Eloc-0.5*sum(B**2,4)
  call temperature(rholoc,Eloc,Tloc,nxmin(1),nxmax(1),nxmin(2),nxmax(2),nxmin(3),nxmax(3))
  Eloc = Eloc + 0.5*sum(rhouloc**2,4)/rholoc
  if(allocated(B)) Eloc = Eloc + 0.5*sum(B**2,4)

  ! Compute kappa
  if(verbose) write(*,*) 'Compute kappa'

  call cal_kappa_dif(rholoc,Tloc,kappa_abs,kappa_sca,nxmin(1),nxmax(1),nxmin(2),nxmax(2),nxmin(3),nxmax(3))

  if(verbose) write(*,*) 'Compute cv'

  call heat_capacity(rholoc,Tloc,cv,nxmin(1),nxmax(1),nxmin(2),nxmax(2),nxmin(3),nxmax(3))
    
  ! Compute k_gradE
  do idim = 1,ndim
  
     do k = 1,nnz+kron(idim,3)
     do j = 1,nny+kron(idim,2)
     do i = 1,nnx+kron(idim,1)

        im1 = i - kron(1,idim) ; jm1 = j - kron(2,idim) ; km1 = k - kron(3,idim)

        coeff = 1.
        if(idim.eq.1) then
           id = i
        else if(idim.eq.2) then
           id = j
           if (geom_dir(2) == 'phi_c' .or. geom_dir(2) == 'the_s') coeff = 0.5*(xloc(i-1,1)+xloc(i,1))
        else if(idim.eq.3) then
           id = k
           if (geom_dir(3) == 'phi_s') coeff = 0.5*(xloc(i-1,1)+xloc(i,1))*sin(0.5*(xloc(j-1,2)+xloc(j,2)))
        endif

        sig = 0.5*(kappa_abs(im1,jm1,km1)*rho(im1,jm1,km1) + kappa_abs(i,j,k)*rho(i,j,k)) 

        !FLD
        xx = max(1.e-7,abs( (Er_difloc(i,j,k) - Er_difloc(im1,jm1,km1)) / &
             (0.5*(Er_difloc(i,j,k) + Er_difloc(im1,jm1,km1))) ) )
        xx = xx/(0.5*(xloc(id+1,idim)-xloc(id-1,idim))*coeff) / sig

        k_gradE(i,j,k,idim) = c/sig * (Er_difloc(i,j,k) - Er_difloc(im1,jm1,km1)) / &
                              (0.5*(xloc(id+1,idim)-xloc(id-1,idim))*coeff)*lambda_fld(xx)

     enddo
     enddo
     enddo

  enddo

  if(verbose) write(*,*) 'Update'

  ! Update Er_dif and T (source term is treated implicitly)
  do k = 1,nnz
     do j = 1,nny
        do i = 1,nnx
        
           do idim  = 1,ndim

              ip1 = i + kron(1,idim) ; jp1 = j + kron(2,idim) ; kp1 = k + kron(3,idim)
              
              flux_ener_tot  = k_gradE(ip1,jp1,kp1,idim)*ds(ip1,jp1,kp1,idim) -  k_gradE(i,j,k,idim)*ds(i,j,k,idim)

           enddo

           omega = kappa_abs(i,j,k)*rholoc(i,j,k)*c*dt
           den   = cv(i,j,k)+omega*derive_source_dif(Tloc(i,j,k))

           Er_difloc(i,j,k) = Er_difloc(i,j,k) + flux_ener_tot * dt / dv(i,j,k) + cv(i,j,k) * &
                              omega * source_dif(Tloc(i,j,k)) / den
                              
           Er_difloc(i,j,k) = Er_difloc(i,j,k) / (1.+omega*cv(i,j,k)/den)
           
                Tloc(i,j,k) = (cv(i,j,k)*Tloc(i,j,k)+omega*(Er_difloc(i,j,k)-source_dif(Tloc(i,j,k))+ &
                              derive_source_dif(Tloc(i,j,k))*Tloc(i,j,k))) / den
                              
        enddo
     enddo
  enddo

  ! Update internal energy
  call energy(rholoc,Tloc,Eloc,nxmin(1),nxmax(1),nxmin(2),nxmax(2),nxmin(3),nxmax(3))
  Eloc = Eloc + 0.5*sum(rhouloc**2,4)/rholoc
  if(allocated(B)) Eloc = Eloc + 0.5*sum(B**2,4)

  ! Fill arrays with updated sub-arrays
  E     (imin:imax,jmin:jmax,kmin:kmax) = Eloc     (1:nnx,1:nny,1:nnz)
  Er_dif(imin:imax,jmin:jmax,kmin:kmax) = Er_difloc(1:nnx,1:nny,1:nnz)

  ! Array deallocation
  deallocate(dx,dxc)
  deallocate(ds,dv)
  deallocate(xloc,xcloc)
  deallocate( rholoc, Eloc)
  deallocate(rhouloc)
  deallocate(Tloc, k_gradE,kappa_abs,kappa_sca)
  deallocate(Er_difloc)

  if (verbose) write(*,*) 'End of diffusion_ex'
  
  return

end subroutine diffusion_ex

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Function LAMBDA_FLD
!
!> This function computes the FLD limiter.
!<
function lambda_fld(x)

  use diffusion, only : fld_limiter

  implicit none

  real :: x,lambda_fld

  select case (fld_limiter)

  case ('minerbo')

     if(x .le. 3.0/2.0) then
        lambda_fld = 2.0/(3.0 + sqrt(9.0 + 12.0*x**2))
     else
        lambda_fld = 1.0/(1.0 + x + sqrt(1.0 + 2.0*x))
     endif

  case ('levermore')

     lambda_fld = (1.0/tanh(x) - 1.0/x) / x

  case ('none')

     lambda_fld = 1.0/3.0

  end select

end function lambda_fld