!> \file
!! Contains subroutine inversion_gmres()
!<

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Subroutine INVERSION_GMRES
!
!> Performs the GMRES matrix inversion.
!<
subroutine inversion_gmres

  use parameters
  use divers_ray, only : deja_converge,matrix_inv_solver
  use tabgmres
  use VarGmres
  use para

  implicit none

  integer, dimension(3) :: info
  integer               :: nloc
  integer               :: i,j,k,ivar

  deja_converge = .false.

  nloc = nx(1)*nx(2)*nx(3)*nvar_ray

  call mat_prod3d(xin,xout)

  do i=1,nx(1)
     do j=1,nx(2)
        do k=1,nx(3)
           do ivar=1,nvar_ray
              work((i+(j-1)*nx(1)+(k-1)*nx(1)*nx(2)-1)*nvar_ray+ivar) = xin( (i-imin+(j-jmin)*(imax-imin+1)+&
                   (k-kmin)*(imax-imin+1)*(jmax-jmin+1)-1)*nvar_ray+ivar )
           enddo
        enddo
     enddo
  enddo
  work(nloc+1:2*nloc) = B(1:nloc)

  call gmres(nx_glob(1)*nx_glob(2)*nx_glob(3)*nvar_ray,restart,nloc,work,nx(1),nx(2),nx(3),&
             nvar_ray,imin,imax,jmin,jmax,kmin,kmax,workred,lwork,cntl,icntl,info)

  deja_converge = .true.

  if (info(1).ne.0 ) then
     if ( mype == 0) then
        if (info(1)==-4) then
           if(matrix_inv_solver=='GMRES-GS'.or.matrix_inv_solver=='yoyo') then
              write(*,*) 'GMRES did not converge, hence switch to Gauss-Seidel...'
              deja_converge=.false.
           else
              write(*,*) '*** Convergence failure GMRES ***'
              read(*,*)
           endif
        else
           write(*,*) '*** info *** = ',info(1)
        endif
     endif
  endif

  xout(1:nloc) = work(1:nloc)
  
  return

end subroutine inversion_gmres
