!> \file
!! Contains module communication_ray and subroutines
!! allocate_array_com_ray(), communications_ray(),
!! com_1D_ray(), com_2D_ray() and com_3D_ray()
!<

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Module COMMUNICATION_RAY
!
!> Contains the communication arrays for the M1 radiative
!! transfer.
!<
module communication_ray
  implicit none
  real, dimension(:,:,:,:), allocatable :: face1_e        !< Face 1 e
  real, dimension(:,:,:,:), allocatable :: face2_e        !< Face 2 e
  real, dimension(:,:,:,:), allocatable :: face3_e        !< Face 3 e
  real, dimension(:,:,:,:), allocatable :: face4_e        !< Face 4 e
  real, dimension(:,:,:,:), allocatable :: face5_e        !< Face 5 e
  real, dimension(:,:,:,:), allocatable :: face6_e        !< Face 6 e
  real, dimension(:,:,:,:), allocatable :: face1_r        !< Face 1 r
  real, dimension(:,:,:,:), allocatable :: face2_r        !< Face 2 r
  real, dimension(:,:,:,:), allocatable :: face3_r        !< Face 3 r
  real, dimension(:,:,:,:), allocatable :: face4_r        !< Face 4 r
  real, dimension(:,:,:,:), allocatable :: face5_r        !< Face 5 r
  real, dimension(:,:,:,:), allocatable :: face6_r        !< Face 6 r
  real, dimension(:,:,:,:), allocatable :: ar13_e         !< Edge 1 e
  real, dimension(:,:,:,:), allocatable :: ar14_e         !< Edge 2 e
  real, dimension(:,:,:,:), allocatable :: ar23_e         !< Edge 3 e
  real, dimension(:,:,:,:), allocatable :: ar24_e         !< Edge 4 e
  real, dimension(:,:,:,:), allocatable :: ar15_e         !< Edge 5 e
  real, dimension(:,:,:,:), allocatable :: ar25_e         !< Edge 6 e
  real, dimension(:,:,:,:), allocatable :: ar35_e         !< Edge 7 e
  real, dimension(:,:,:,:), allocatable :: ar45_e         !< Edge 8 e
  real, dimension(:,:,:,:), allocatable :: ar16_e         !< Edge 9 e
  real, dimension(:,:,:,:), allocatable :: ar26_e         !< Edge 10 e
  real, dimension(:,:,:,:), allocatable :: ar36_e         !< Edge 11 e
  real, dimension(:,:,:,:), allocatable :: ar46_e         !< Edge 12 e
  real, dimension(:,:,:,:), allocatable :: ar13_r         !< Edge 1 r
  real, dimension(:,:,:,:), allocatable :: ar14_r         !< Edge 2 r
  real, dimension(:,:,:,:), allocatable :: ar23_r         !< Edge 3 r
  real, dimension(:,:,:,:), allocatable :: ar24_r         !< Edge 4 r
  real, dimension(:,:,:,:), allocatable :: ar15_r         !< Edge 5 r
  real, dimension(:,:,:,:), allocatable :: ar25_r         !< Edge 6 r
  real, dimension(:,:,:,:), allocatable :: ar35_r         !< Edge 7 r
  real, dimension(:,:,:,:), allocatable :: ar45_r         !< Edge 8 r
  real, dimension(:,:,:,:), allocatable :: ar16_r         !< Edge 9 r
  real, dimension(:,:,:,:), allocatable :: ar26_r         !< Edge 10 r
  real, dimension(:,:,:,:), allocatable :: ar36_r         !< Edge 11 r
  real, dimension(:,:,:,:), allocatable :: ar46_r         !< Edge 12 r
  real, dimension(:,:,:,:), allocatable :: coin13_e       !< Corner 2D 1 e
  real, dimension(:,:,:,:), allocatable :: coin14_e       !< Corner 2D 2 e
  real, dimension(:,:,:,:), allocatable :: coin23_e       !< Corner 2D 3 e
  real, dimension(:,:,:,:), allocatable :: coin24_e       !< Corner 2D 4 e
  real, dimension(:,:,:,:), allocatable :: coin13_r       !< Corner 2D 1 r
  real, dimension(:,:,:,:), allocatable :: coin14_r       !< Corner 2D 2 r
  real, dimension(:,:,:,:), allocatable :: coin23_r       !< Corner 2D 3 r
  real, dimension(:,:,:,:), allocatable :: coin24_r       !< Corner 2D 4 r
  real, dimension(:,:,:,:), allocatable :: coin135_e      !< Corner 3D 1 e
  real, dimension(:,:,:,:), allocatable :: coin136_e      !< Corner 3D 2 e
  real, dimension(:,:,:,:), allocatable :: coin145_e      !< Corner 3D 3 e
  real, dimension(:,:,:,:), allocatable :: coin146_e      !< Corner 3D 4 e
  real, dimension(:,:,:,:), allocatable :: coin235_e      !< Corner 3D 5 e
  real, dimension(:,:,:,:), allocatable :: coin236_e      !< Corner 3D 6 e
  real, dimension(:,:,:,:), allocatable :: coin245_e      !< Corner 3D 7 e
  real, dimension(:,:,:,:), allocatable :: coin246_e      !< Corner 3D 8 e
  real, dimension(:,:,:,:), allocatable :: coin135_r      !< Corner 3D 1 r
  real, dimension(:,:,:,:), allocatable :: coin136_r      !< Corner 3D 2 r
  real, dimension(:,:,:,:), allocatable :: coin145_r      !< Corner 3D 3 r
  real, dimension(:,:,:,:), allocatable :: coin146_r      !< Corner 3D 4 r
  real, dimension(:,:,:,:), allocatable :: coin235_r      !< Corner 3D 5 r
  real, dimension(:,:,:,:), allocatable :: coin236_r      !< Corner 3D 6 r
  real, dimension(:,:,:,:), allocatable :: coin245_r      !< Corner 3D 7 r
  real, dimension(:,:,:,:), allocatable :: coin246_r      !< Corner 3D 8 r
end module communication_ray

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Subroutine ALLOCATE_ARRAY_COM_RAY
!
!> Allocates the communication arrays for the M1 radiative
!! transfer.
!<
subroutine allocate_array_com_ray

  use communication_ray
  use parameters
  use para

  implicit none

  integer :: nnx,nny,nnz
  
  nnx = nx(1) ; nny = nx(2) ; nnz = nx(3)

  if (ndim == 1) then
     allocate(face1_e(Nbuf,1,1,nvar_ray), face2_e(Nbuf,1,1,nvar_ray))
     allocate(face1_r(Nbuf,1,1,nvar_ray), face2_r(Nbuf,1,1,nvar_ray))
  endif
     
  if (ndim == 2) then
     allocate(face1_e(Nbuf,1:nny,1,nvar_ray), face2_e(Nbuf,1:nny,1,nvar_ray))
     allocate(face1_r(Nbuf,1:nny,1,nvar_ray), face2_r(Nbuf,1:nny,1,nvar_ray))
     
     allocate(face3_e(1:nnx,Nbuf,1,nvar_ray), face4_e(1:nnx,Nbuf,1,nvar_ray))
     allocate(face3_r(1:nnx,Nbuf,1,nvar_ray), face4_r(1:nnx,Nbuf,1,nvar_ray))
     
     allocate(coin13_e(Nbuf,Nbuf,1,nvar_ray),coin14_e(Nbuf,Nbuf,1,nvar_ray),coin23_e(Nbuf,Nbuf,1,nvar_ray),coin24_e(Nbuf,Nbuf,1,nvar_ray))
     allocate(coin13_r(Nbuf,Nbuf,1,nvar_ray),coin14_r(Nbuf,Nbuf,1,nvar_ray),coin23_r(Nbuf,Nbuf,1,nvar_ray),coin24_r(Nbuf,Nbuf,1,nvar_ray))
  endif
  
  if (ndim == 3) then
     allocate(face1_e(Nbuf,1:nny,1:nnz,nvar_ray), face2_e(Nbuf,1:nny,1:nnz,nvar_ray))
     allocate(face1_r(Nbuf,1:nny,1:nnz,nvar_ray), face2_r(Nbuf,1:nny,1:nnz,nvar_ray))
     
     allocate(face3_e(1:nnx,Nbuf,1:nnz,nvar_ray), face4_e(1:nnx,Nbuf,1:nnz,nvar_ray))
     allocate(face3_r(1:nnx,Nbuf,1:nnz,nvar_ray), face4_r(1:nnx,Nbuf,1:nnz,nvar_ray))
     
     allocate(face5_e(1:nnx,1:nny,Nbuf,nvar_ray), face6_e(1:nnx,1:nny,Nbuf,nvar_ray))
     allocate(face5_r(1:nnx,1:nny,Nbuf,nvar_ray), face6_r(1:nnx,1:nny,Nbuf,nvar_ray))
     
     
     allocate(ar13_e(Nbuf,Nbuf,1:nnz,nvar_ray), ar14_e(Nbuf,Nbuf,1:nnz,nvar_ray), ar23_e(Nbuf,Nbuf,1:nnz,nvar_ray), ar24_e(Nbuf,Nbuf,1:nnz,nvar_ray))
     allocate(ar13_r(Nbuf,Nbuf,1:nnz,nvar_ray), ar14_r(Nbuf,Nbuf,1:nnz,nvar_ray), ar23_r(Nbuf,Nbuf,1:nnz,nvar_ray), ar24_r(Nbuf,Nbuf,1:nnz,nvar_ray))
     
     allocate(ar15_e(Nbuf,1:nny,Nbuf,nvar_ray), ar25_e(Nbuf,1:nny,Nbuf,nvar_ray), ar16_e(Nbuf,1:nny,Nbuf,nvar_ray), ar26_e(Nbuf,1:nny,Nbuf,nvar_ray))
     allocate(ar15_r(Nbuf,1:nny,Nbuf,nvar_ray), ar25_r(Nbuf,1:nny,Nbuf,nvar_ray), ar16_r(Nbuf,1:nny,Nbuf,nvar_ray), ar26_r(Nbuf,1:nny,Nbuf,nvar_ray))
     
     allocate(ar35_e(1:nnx,Nbuf,Nbuf,nvar_ray), ar45_e(1:nnx,Nbuf,Nbuf,nvar_ray), ar36_e(1:nnx,Nbuf,Nbuf,nvar_ray), ar46_e(1:nnx,Nbuf,Nbuf,nvar_ray))
     allocate(ar35_r(1:nnx,Nbuf,Nbuf,nvar_ray), ar45_r(1:nnx,Nbuf,Nbuf,nvar_ray), ar36_r(1:nnx,Nbuf,Nbuf,nvar_ray), ar46_r(1:nnx,Nbuf,Nbuf,nvar_ray))
     
     
     allocate(coin135_e(Nbuf,Nbuf,Nbuf,nvar_ray),coin136_e(Nbuf,Nbuf,Nbuf,nvar_ray),coin145_e(Nbuf,Nbuf,Nbuf,nvar_ray),coin146_e(Nbuf,Nbuf,Nbuf,nvar_ray))
     allocate(coin135_r(Nbuf,Nbuf,Nbuf,nvar_ray),coin136_r(Nbuf,Nbuf,Nbuf,nvar_ray),coin145_r(Nbuf,Nbuf,Nbuf,nvar_ray),coin146_r(Nbuf,Nbuf,Nbuf,nvar_ray))
     allocate(coin235_e(Nbuf,Nbuf,Nbuf,nvar_ray),coin236_e(Nbuf,Nbuf,Nbuf,nvar_ray),coin245_e(Nbuf,Nbuf,Nbuf,nvar_ray),coin246_e(Nbuf,Nbuf,Nbuf,nvar_ray))
     allocate(coin235_r(Nbuf,Nbuf,Nbuf,nvar_ray),coin236_r(Nbuf,Nbuf,Nbuf,nvar_ray),coin245_r(Nbuf,Nbuf,Nbuf,nvar_ray),coin246_r(Nbuf,Nbuf,Nbuf,nvar_ray))
  endif
  
  return

end subroutine allocate_array_com_ray

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Subroutine COMMUNICATIONS_RAY
!
!> Perform communications and return cputime used.
!<
subroutine communications_ray(tcomm)

  use parameters, only : ndim

  implicit none
  
#if PARA==1
  include 'mpif.h'
  integer :: ierr
  real    :: tr1,tr2
#endif

  real :: tcomm

#if PARA==1
  tr1 = MPI_Wtime(ierr)
  if (ndim.eq.1) then
     call com_1D_ray
  else if (ndim.eq.2) then
     call com_2D_ray
  else if (ndim.eq.3) then
     call com_3D_ray
  endif
  tr2 = MPI_Wtime(ierr)     
  tcomm = tcomm + tr2-tr1
#endif

  return
  
end subroutine communications_ray

!###########################################################
!###########################################################
!###########################################################
!###########################################################

#if PARA==1

!  Subroutine COM_1D_RAY
!
!> Performs 1D M1 radiative transfer communications.
!<
subroutine com_1D_ray

  use communication_ray
  use parameters
  use para
  use varray
  use var              , only : Tgaz
  use divers           , only : verbose

  implicit none

  include 'mpif.h'

  integer, parameter                    :: LEFT=1,RIGHT=2
  integer                               :: icount,ierr
  integer                               :: destination, emetteur
  integer, dimension(4  )               :: Req
  integer, dimension(MPI_STATUS_SIZE,4) :: Stat

  integer :: nnx

  if(verbose) write(*,*) 'Start of com_1D_ray'

  nnx = nx(1) 

  icount   = nvar_ray*Nbuf
  Req(1:4) = MPI_REQUEST_NULL


  face1_e(1:Nbuf,1,1,1) = Eray(1:Nbuf,1,1  ) 
  face1_e(1:Nbuf,1,1,2) = Fray(1:Nbuf,1,1,1) 
  face1_e(1:Nbuf,1,1,3) = Tgaz(1:Nbuf,1,1  ) 

  face2_e(1:Nbuf,1,1,1) = Eray(nnx+1-Nbuf:nnx,1,1  ) 
  face2_e(1:Nbuf,1,1,2) = Fray(nnx+1-Nbuf:nnx,1,1,1) 
  face2_e(1:Nbuf,1,1,3) = Tgaz(nnx+1-Nbuf:nnx,1,1  ) 

  ! send data
            
  destination = voisin(-1,0,0)
  call MPI_Isend(face1_e,icount,MPI_DOUBLE_PRECISION,destination,LEFT ,MPI_Comm_world,Req(1),ierr)

  destination = voisin(+1,0,0)
  call MPI_Isend(face2_e,icount,MPI_DOUBLE_PRECISION,destination,RIGHT,MPI_COMM_WORLD,Req(2),ierr)

  ! receive data
  
  emetteur = voisin(+1,0,0)
  call MPI_Irecv(face2_r,icount,MPI_DOUBLE_PRECISION,emetteur   ,LEFT ,MPI_COMM_WORLD,Req(3),ierr)
  
  emetteur = voisin(-1,0,0)
  call MPI_Irecv(face1_r,icount,MPI_DOUBLE_PRECISION,emetteur   ,RIGHT,MPI_COMM_WORLD,Req(4),ierr)
  
  call MPI_Waitall(4,Req,Stat,ierr)


  Eray(1-Nbuf:0,1,1  ) = face1_r(1:Nbuf,1,1,1)
  Fray(1-Nbuf:0,1,1,1) = face1_r(1:Nbuf,1,1,2)
  Tgaz(1-Nbuf:0,1,1  ) = face1_r(1:Nbuf,1,1,3)

  Eray(nnx+1:nnx+Nbuf,1,1  ) = face2_r(1:Nbuf,1,1,1)
  Fray(nnx+1:nnx+Nbuf,1,1,1) = face2_r(1:Nbuf,1,1,2)
  Tgaz(nnx+1:nnx+Nbuf,1,1  ) = face2_r(1:Nbuf,1,1,3)
  
  if(verbose) write(*,*) 'End of com_1D_ray'
  
  return
  
end subroutine com_1D_ray

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Subroutine COM_2D_RAY
!
!> Performs 2D M1 radiative transfer communications.
!<
subroutine com_2D_ray

  use communication_ray
  use parameters
  use para
  use varray
  use var              , only : Tgaz
  use divers           , only : verbose

  implicit none

  include 'mpif.h'

  integer, parameter                     :: LEFT=1,RIGHT=2,DOWN=3, UP=4, RD=13, RU=14, LD=23, LU = 24
  integer                                :: ierr
  integer                                :: ic_f12,ic_f34,ic_coin
  integer                                :: destination, emetteur
  integer, dimension(16  )               :: Req
  integer, dimension(MPI_STATUS_SIZE,16) :: Stat

  integer :: nnx, nny

  if(verbose) write(*,*) 'Start of com_2D_ray'

  nnx = nx(1) ; nny = nx(2) 

  ic_f12  = nvar_ray*Nbuf*nny
  ic_f34  = nvar_ray*Nbuf*nnx
  ic_coin = nvar_ray*Nbuf*Nbuf

  Req(1:16) = MPI_REQUEST_NULL

  ! 4 faces
  face1_e(1:Nbuf,1:nny,1,1) = Eray(1:Nbuf,1:nny,1  ) 
  face1_e(1:Nbuf,1:nny,1,2) = Fray(1:Nbuf,1:nny,1,1) 
  face1_e(1:Nbuf,1:nny,1,3) = Fray(1:Nbuf,1:nny,1,2) 
  face1_e(1:Nbuf,1:nny,1,4) = Tgaz(1:Nbuf,1:nny,1  ) 

  face2_e(1:Nbuf,1:nny,1,1) = Eray(nnx+1-Nbuf:nnx,1:nny,1  ) 
  face2_e(1:Nbuf,1:nny,1,2) = Fray(nnx+1-Nbuf:nnx,1:nny,1,1) 
  face2_e(1:Nbuf,1:nny,1,3) = Fray(nnx+1-Nbuf:nnx,1:nny,1,2) 
  face2_e(1:Nbuf,1:nny,1,4) = Tgaz(nnx+1-Nbuf:nnx,1:nny,1  ) 

  face3_e(1:nnx,1:Nbuf,1,1) = Eray(1:nnx,1:Nbuf,1  ) 
  face3_e(1:nnx,1:Nbuf,1,2) = Fray(1:nnx,1:Nbuf,1,1) 
  face3_e(1:nnx,1:Nbuf,1,3) = Fray(1:nnx,1:Nbuf,1,2) 
  face3_e(1:nnx,1:Nbuf,1,4) = Tgaz(1:nnx,1:Nbuf,1  ) 

  face4_e(1:nnx,1:Nbuf,1,1) = Eray(1:nnx,nny+1-Nbuf:nny,1  ) 
  face4_e(1:nnx,1:Nbuf,1,2) = Fray(1:nnx,nny+1-Nbuf:nny,1,1) 
  face4_e(1:nnx,1:Nbuf,1,3) = Fray(1:nnx,nny+1-Nbuf:nny,1,2) 
  face4_e(1:nnx,1:Nbuf,1,4) = Tgaz(1:nnx,nny+1-Nbuf:nny,1  ) 
  

  ! 4 corners
  coin13_e(1:Nbuf,1:Nbuf,1,1) = Eray(1:Nbuf,1:Nbuf,1  ) 
  coin13_e(1:Nbuf,1:Nbuf,1,2) = Fray(1:Nbuf,1:Nbuf,1,1) 
  coin13_e(1:Nbuf,1:Nbuf,1,3) = Fray(1:Nbuf,1:Nbuf,1,2) 
  coin13_e(1:Nbuf,1:Nbuf,1,4) = Tgaz(1:Nbuf,1:Nbuf,1  ) 

  coin23_e(1:Nbuf,1:Nbuf,1,1) = Eray(nnx+1-Nbuf:nnx,1:Nbuf,1  ) 
  coin23_e(1:Nbuf,1:Nbuf,1,2) = Fray(nnx+1-Nbuf:nnx,1:Nbuf,1,1) 
  coin23_e(1:Nbuf,1:Nbuf,1,3) = Fray(nnx+1-Nbuf:nnx,1:Nbuf,1,2)
  coin23_e(1:Nbuf,1:Nbuf,1,4) = Tgaz(nnx+1-Nbuf:nnx,1:Nbuf,1  ) 
  
  coin14_e(1:Nbuf,1:Nbuf,1,1) = Eray(1:Nbuf,nny+1-Nbuf:nny,1  ) 
  coin14_e(1:Nbuf,1:Nbuf,1,2) = Fray(1:Nbuf,nny+1-Nbuf:nny,1,1) 
  coin14_e(1:Nbuf,1:Nbuf,1,3) = Fray(1:Nbuf,nny+1-Nbuf:nny,1,2) 
  coin14_e(1:Nbuf,1:Nbuf,1,4) = Tgaz(1:Nbuf,nny+1-Nbuf:nny,1  ) 

  coin24_e(1:Nbuf,1:Nbuf,1,1) = Eray(nnx+1-Nbuf:nnx,nny+1-Nbuf:nny,1  ) 
  coin24_e(1:Nbuf,1:Nbuf,1,2) = Fray(nnx+1-Nbuf:nnx,nny+1-Nbuf:nny,1,1) 
  coin24_e(1:Nbuf,1:Nbuf,1,3) = Fray(nnx+1-Nbuf:nnx,nny+1-Nbuf:nny,1,2) 
  coin24_e(1:Nbuf,1:Nbuf,1,4) = Tgaz(nnx+1-Nbuf:nnx,nny+1-Nbuf:nny,1  ) 


  ! send data
            
  destination = voisin(-1, 0,0)
  call MPI_Isend(face1_e ,ic_f12 ,MPI_DOUBLE_PRECISION,destination,LEFT ,MPI_Comm_world,Req( 1),ierr)

  destination = voisin(+1, 0,0)
  call MPI_Isend(face2_e ,ic_f12 ,MPI_DOUBLE_PRECISION,destination,RIGHT,MPI_COMM_WORLD,Req( 2),ierr)

  destination = voisin( 0,-1,0)
  call MPI_Isend(face3_e ,ic_f34 ,MPI_DOUBLE_PRECISION,destination,DOWN ,MPI_COMM_WORLD,Req( 3),ierr)

  destination = voisin( 0,+1,0)
  call MPI_Isend(face4_e ,ic_f34 ,MPI_DOUBLE_PRECISION,destination,UP   ,MPI_COMM_WORLD,Req( 4),ierr)

  destination = voisin(-1,-1,0)
  call MPI_Isend(coin13_e,ic_coin,MPI_DOUBLE_PRECISION,destination,LD   ,MPI_COMM_WORLD,Req( 5),ierr)

  destination = voisin(-1,+1,0)
  call MPI_Isend(coin14_e,ic_coin,MPI_DOUBLE_PRECISION,destination,LU   ,MPI_COMM_WORLD,Req( 6),ierr)

  destination = voisin(+1,-1,0)
  call MPI_Isend(coin23_e,ic_coin,MPI_DOUBLE_PRECISION,destination,RD   ,MPI_COMM_WORLD,Req( 7),ierr)

  destination = voisin(+1,+1,0)
  call MPI_Isend(coin24_e,ic_coin,MPI_DOUBLE_PRECISION,destination,RU   ,MPI_COMM_WORLD,Req( 8),ierr)


  ! receive data
  
  emetteur = voisin(+1, 0,0)
  call MPI_Irecv(face2_r ,ic_f12 ,MPI_DOUBLE_PRECISION,emetteur   ,LEFT ,MPI_COMM_WORLD,Req( 9),ierr)
  
  emetteur = voisin(-1, 0,0)
  call MPI_Irecv(face1_r ,ic_f12 ,MPI_DOUBLE_PRECISION,emetteur   ,RIGHT,MPI_COMM_WORLD,Req(10),ierr)
  
  emetteur = voisin( 0,+1,0)
  call MPI_Irecv(face4_r ,ic_f34 ,MPI_DOUBLE_PRECISION,emetteur   ,DOWN ,MPI_COMM_WORLD,Req(11),ierr)
  
  emetteur = voisin( 0,-1,0)
  call MPI_Irecv(face3_r ,ic_f34 ,MPI_DOUBLE_PRECISION,emetteur   ,UP   ,MPI_COMM_WORLD,Req(12),ierr)
  

  emetteur = voisin(+1,+1,0)
  call MPI_Irecv(coin24_r,ic_coin,MPI_DOUBLE_PRECISION,emetteur   ,LD   ,MPI_COMM_WORLD,Req(13),ierr)
  
  emetteur = voisin(+1,-1,0)
  call MPI_Irecv(coin23_r,ic_coin,MPI_DOUBLE_PRECISION,emetteur   ,LU   ,MPI_COMM_WORLD,Req(14),ierr)

  emetteur = voisin(-1,+1,0)
  call MPI_Irecv(coin14_r,ic_coin,MPI_DOUBLE_PRECISION,emetteur   ,RD   ,MPI_COMM_WORLD,Req(15),ierr)

  emetteur = voisin(-1,-1,0)
  call MPI_Irecv(coin13_r,ic_coin,MPI_DOUBLE_PRECISION,emetteur   ,RU   ,MPI_COMM_WORLD,Req(16),ierr)


  call MPI_Waitall(16,Req,Stat,ierr)


  ! 4 faces
  Eray(1-Nbuf:0,1:nny,1  ) = face1_r(1:Nbuf,1:nny,1,1)
  Fray(1-Nbuf:0,1:nny,1,1) = face1_r(1:Nbuf,1:nny,1,2)
  Fray(1-Nbuf:0,1:nny,1,2) = face1_r(1:Nbuf,1:nny,1,3)
  Tgaz(1-Nbuf:0,1:nny,1  ) = face1_r(1:Nbuf,1:nny,1,4)

  Eray(nnx+1:nnx+Nbuf,1:nny,1  ) = face2_r(1:Nbuf,1:nny,1,1)
  Fray(nnx+1:nnx+Nbuf,1:nny,1,1) = face2_r(1:Nbuf,1:nny,1,2)
  Fray(nnx+1:nnx+Nbuf,1:nny,1,2) = face2_r(1:Nbuf,1:nny,1,3)
  Tgaz(nnx+1:nnx+Nbuf,1:nny,1  ) = face2_r(1:Nbuf,1:nny,1,4)

  Eray(1:nnx,1-Nbuf:0,1  ) = face3_r(1:nnx,1:Nbuf,1,1)
  Fray(1:nnx,1-Nbuf:0,1,1) = face3_r(1:nnx,1:Nbuf,1,2)
  Fray(1:nnx,1-Nbuf:0,1,2) = face3_r(1:nnx,1:Nbuf,1,3)
  Tgaz(1:nnx,1-Nbuf:0,1  ) = face3_r(1:nnx,1:Nbuf,1,4)

  Eray(1:nnx,nny+1:nny+Nbuf,1  ) = face4_r(1:nnx,1:Nbuf,1,1)
  Fray(1:nnx,nny+1:nny+Nbuf,1,1) = face4_r(1:nnx,1:Nbuf,1,2)
  Fray(1:nnx,nny+1:nny+Nbuf,1,2) = face4_r(1:nnx,1:Nbuf,1,3)
  Tgaz(1:nnx,nny+1:nny+Nbuf,1  ) = face4_r(1:nnx,1:Nbuf,1,4)
  
  ! 4 corners
  Eray(1-Nbuf:0,1-Nbuf:0,1  ) = coin13_r(1:Nbuf,1:Nbuf,1,1)
  Fray(1-Nbuf:0,1-Nbuf:0,1,1) = coin13_r(1:Nbuf,1:Nbuf,1,2)
  Fray(1-Nbuf:0,1-Nbuf:0,1,2) = coin13_r(1:Nbuf,1:Nbuf,1,3)
  Tgaz(1-Nbuf:0,1-Nbuf:0,1  ) = coin13_r(1:Nbuf,1:Nbuf,1,4)

  Eray(1-Nbuf:0,nny+1:nny+Nbuf,1  ) = coin14_r(1:Nbuf,1:Nbuf,1,1)
  Fray(1-Nbuf:0,nny+1:nny+Nbuf,1,1) = coin14_r(1:Nbuf,1:Nbuf,1,2)
  Fray(1-Nbuf:0,nny+1:nny+Nbuf,1,2) = coin14_r(1:Nbuf,1:Nbuf,1,3)
  Tgaz(1-Nbuf:0,nny+1:nny+Nbuf,1  ) = coin14_r(1:Nbuf,1:Nbuf,1,4)

  Eray(nnx+1:nnx+Nbuf,1-Nbuf:0,1  ) = coin23_r(1:Nbuf,1:Nbuf,1,1)
  Fray(nnx+1:nnx+Nbuf,1-Nbuf:0,1,1) = coin23_r(1:Nbuf,1:Nbuf,1,2)
  Fray(nnx+1:nnx+Nbuf,1-Nbuf:0,1,2) = coin23_r(1:Nbuf,1:Nbuf,1,3)
  Tgaz(nnx+1:nnx+Nbuf,1-Nbuf:0,1  ) = coin23_r(1:Nbuf,1:Nbuf,1,4)

  Eray(nnx+1:nnx+Nbuf,nny+1:nny+Nbuf,1  ) = coin24_r(1:Nbuf,1:Nbuf,1,1)
  Fray(nnx+1:nnx+Nbuf,nny+1:nny+Nbuf,1,1) = coin24_r(1:Nbuf,1:Nbuf,1,2)
  Fray(nnx+1:nnx+Nbuf,nny+1:nny+Nbuf,1,2) = coin24_r(1:Nbuf,1:Nbuf,1,3)
  Tgaz(nnx+1:nnx+Nbuf,nny+1:nny+Nbuf,1  ) = coin24_r(1:Nbuf,1:Nbuf,1,4)
  
  if(verbose) write(*,*) 'End of com_2D_ray'
  
  return
  
end subroutine com_2D_ray

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Subroutine COM_3D_RAY
!
!> Performs 3D M1 radiative transfer communications.
!<
subroutine com_3D_ray

  use communication_ray
  use parameters
  use para
  use varray
  use var              , only : Tgaz
  use divers           , only : verbose

  implicit none

  include 'mpif.h'

  integer, parameter                     :: F1=1, F2=2, F3=3, F4=4, F5=5, F6=6
  integer, parameter                     :: AR13=13, AR14=14, AR15=15, AR16=16
  integer, parameter                     :: AR23=23, AR24=24, AR25=25, AR26=26
  integer, parameter                     :: AR35=35, AR36=36
  integer, parameter                     :: AR45=45, AR46=46
  integer, parameter                     :: C135=135, C136=136, C145=145, C146=146
  integer, parameter                     :: C235=235, C236=236, C245=245, C246=246
  integer                                :: ierr
  integer                                :: ic_f12,ic_f34,ic_f56, ic_arx, ic_ary, ic_arz, ic_coin
  integer                                :: destination, emetteur
  integer, dimension(52  )               :: Req
  integer, dimension(MPI_STATUS_SIZE,52) :: Stat

  integer :: nnx, nny, nnz
  
  if(verbose) write(*,*) 'Start of com_3D_ray'

  nnx = nx(1) ; nny = nx(2) ; nnz = nx(3)

  ic_f12  = nvar_ray*Nbuf*nny*nnz
  ic_f34  = nvar_ray*Nbuf*nnx*nnz
  ic_f56  = nvar_ray*Nbuf*nnx*nny

  ic_arx  = nvar_ray*Nbuf*Nbuf*nnx
  ic_ary  = nvar_ray*Nbuf*Nbuf*nny
  ic_arz  = nvar_ray*Nbuf*Nbuf*nnz

  ic_coin = nvar_ray*Nbuf*Nbuf*Nbuf

  Req(1:52) = MPI_REQUEST_NULL

  ! 6 faces
  face1_e(1:Nbuf,1:nny,1:nnz,1) = Eray(1:Nbuf,1:nny,1:nnz  ) 
  face1_e(1:Nbuf,1:nny,1:nnz,2) = Fray(1:Nbuf,1:nny,1:nnz,1) 
  face1_e(1:Nbuf,1:nny,1:nnz,3) = Fray(1:Nbuf,1:nny,1:nnz,2) 
  face1_e(1:Nbuf,1:nny,1:nnz,4) = Fray(1:Nbuf,1:nny,1:nnz,3) 
  face1_e(1:Nbuf,1:nny,1:nnz,5) = Tgaz(1:Nbuf,1:nny,1:nnz  ) 

  face2_e(1:Nbuf,1:nny,1:nnz,1) = Eray(nnx+1-Nbuf:nnx,1:nny,1:nnz  ) 
  face2_e(1:Nbuf,1:nny,1:nnz,2) = Fray(nnx+1-Nbuf:nnx,1:nny,1:nnz,1) 
  face2_e(1:Nbuf,1:nny,1:nnz,3) = Fray(nnx+1-Nbuf:nnx,1:nny,1:nnz,2) 
  face2_e(1:Nbuf,1:nny,1:nnz,4) = Fray(nnx+1-Nbuf:nnx,1:nny,1:nnz,3) 
  face2_e(1:Nbuf,1:nny,1:nnz,5) = Tgaz(nnx+1-Nbuf:nnx,1:nny,1:nnz  ) 

  face3_e(1:nnx,1:Nbuf,1:nnz,1) = Eray(1:nnx,1:Nbuf,1:nnz  ) 
  face3_e(1:nnx,1:Nbuf,1:nnz,2) = Fray(1:nnx,1:Nbuf,1:nnz,1) 
  face3_e(1:nnx,1:Nbuf,1:nnz,3) = Fray(1:nnx,1:Nbuf,1:nnz,2) 
  face3_e(1:nnx,1:Nbuf,1:nnz,4) = Fray(1:nnx,1:Nbuf,1:nnz,3) 
  face3_e(1:nnx,1:Nbuf,1:nnz,5) = Tgaz(1:nnx,1:Nbuf,1:nnz  ) 

  face4_e(1:nnx,1:Nbuf,1:nnz,1) = Eray(1:nnx,nny+1-Nbuf:nny,1:nnz  ) 
  face4_e(1:nnx,1:Nbuf,1:nnz,2) = Fray(1:nnx,nny+1-Nbuf:nny,1:nnz,1) 
  face4_e(1:nnx,1:Nbuf,1:nnz,3) = Fray(1:nnx,nny+1-Nbuf:nny,1:nnz,2) 
  face4_e(1:nnx,1:Nbuf,1:nnz,4) = Fray(1:nnx,nny+1-Nbuf:nny,1:nnz,3) 
  face4_e(1:nnx,1:Nbuf,1:nnz,5) = Tgaz(1:nnx,nny+1-Nbuf:nny,1:nnz  ) 
  
  face5_e(1:nnx,1:nny,1:Nbuf,1) = Eray(1:nnx,1:nny,1:Nbuf  ) 
  face5_e(1:nnx,1:nny,1:Nbuf,2) = Fray(1:nnx,1:nny,1:Nbuf,1) 
  face5_e(1:nnx,1:nny,1:Nbuf,3) = Fray(1:nnx,1:nny,1:Nbuf,2) 
  face5_e(1:nnx,1:nny,1:Nbuf,4) = Fray(1:nnx,1:nny,1:Nbuf,3) 
  face5_e(1:nnx,1:nny,1:Nbuf,5) = Tgaz(1:nnx,1:nny,1:Nbuf  ) 

  face6_e(1:nnx,1:nny,1:Nbuf,1) = Eray(1:nnx,1:nny,nnz+1-Nbuf:nnz  ) 
  face6_e(1:nnx,1:nny,1:Nbuf,2) = Fray(1:nnx,1:nny,nnz+1-Nbuf:nnz,1) 
  face6_e(1:nnx,1:nny,1:Nbuf,3) = Fray(1:nnx,1:nny,nnz+1-Nbuf:nnz,2) 
  face6_e(1:nnx,1:nny,1:Nbuf,4) = Fray(1:nnx,1:nny,nnz+1-Nbuf:nnz,3) 
  face6_e(1:nnx,1:nny,1:Nbuf,5) = Tgaz(1:nnx,1:nny,nnz+1-Nbuf:nnz  ) 
  
  ! 12 edges
  ar13_e(1:Nbuf, 1:Nbuf, 1:nnz, 1  ) = Eray(1:Nbuf, 1:Nbuf, 1:nnz   )
  ar13_e(1:Nbuf, 1:Nbuf, 1:nnz, 2  ) = Fray(1:Nbuf, 1:Nbuf, 1:nnz, 1)
  ar13_e(1:Nbuf, 1:Nbuf, 1:nnz, 3  ) = Fray(1:Nbuf, 1:Nbuf, 1:nnz, 2)
  ar13_e(1:Nbuf, 1:Nbuf, 1:nnz, 4  ) = Fray(1:Nbuf, 1:Nbuf, 1:nnz, 3)
  ar13_e(1:Nbuf, 1:Nbuf, 1:nnz, 5  ) = Tgaz(1:Nbuf, 1:Nbuf, 1:nnz   )

  ar14_e(1:Nbuf, 1:Nbuf, 1:nnz, 1  ) = Eray(1:Nbuf, nny+1-Nbuf:nny, 1:nnz   )
  ar14_e(1:Nbuf, 1:Nbuf, 1:nnz, 2  ) = Fray(1:Nbuf, nny+1-Nbuf:nny, 1:nnz, 1)
  ar14_e(1:Nbuf, 1:Nbuf, 1:nnz, 3  ) = Fray(1:Nbuf, nny+1-Nbuf:nny, 1:nnz, 2)
  ar14_e(1:Nbuf, 1:Nbuf, 1:nnz, 4  ) = Fray(1:Nbuf, nny+1-Nbuf:nny, 1:nnz, 3)
  ar14_e(1:Nbuf, 1:Nbuf, 1:nnz, 5  ) = Tgaz(1:Nbuf, nny+1-Nbuf:nny, 1:nnz   )

  ar23_e(1:Nbuf, 1:Nbuf, 1:nnz, 1  ) = Eray(nnx+1-Nbuf:nnx, 1:Nbuf, 1:nnz   )
  ar23_e(1:Nbuf, 1:Nbuf, 1:nnz, 2  ) = Fray(nnx+1-Nbuf:nnx, 1:Nbuf, 1:nnz, 1)
  ar23_e(1:Nbuf, 1:Nbuf, 1:nnz, 3  ) = Fray(nnx+1-Nbuf:nnx, 1:Nbuf, 1:nnz, 2)
  ar23_e(1:Nbuf, 1:Nbuf, 1:nnz, 4  ) = Fray(nnx+1-Nbuf:nnx, 1:Nbuf, 1:nnz, 3)
  ar23_e(1:Nbuf, 1:Nbuf, 1:nnz, 5  ) = Tgaz(nnx+1-Nbuf:nnx, 1:Nbuf, 1:nnz   )

  ar24_e(1:Nbuf, 1:Nbuf, 1:nnz, 1  ) = Eray(nnx+1-Nbuf:nnx, nny+1-Nbuf:nny, 1:nnz   )
  ar24_e(1:Nbuf, 1:Nbuf, 1:nnz, 2  ) = Fray(nnx+1-Nbuf:nnx, nny+1-Nbuf:nny, 1:nnz, 1)
  ar24_e(1:Nbuf, 1:Nbuf, 1:nnz, 3  ) = Fray(nnx+1-Nbuf:nnx, nny+1-Nbuf:nny, 1:nnz, 2)
  ar24_e(1:Nbuf, 1:Nbuf, 1:nnz, 4  ) = Fray(nnx+1-Nbuf:nnx, nny+1-Nbuf:nny, 1:nnz, 3)
  ar24_e(1:Nbuf, 1:Nbuf, 1:nnz, 5  ) = Tgaz(nnx+1-Nbuf:nnx, nny+1-Nbuf:nny, 1:nnz   )



  ar15_e(1:Nbuf, 1:nny, 1:Nbuf, 1  ) = Eray(1:Nbuf, 1:nny, 1:Nbuf   )
  ar15_e(1:Nbuf, 1:nny, 1:Nbuf, 2  ) = Fray(1:Nbuf, 1:nny, 1:Nbuf, 1)
  ar15_e(1:Nbuf, 1:nny, 1:Nbuf, 3  ) = Fray(1:Nbuf, 1:nny, 1:Nbuf, 2)
  ar15_e(1:Nbuf, 1:nny, 1:Nbuf, 4  ) = Fray(1:Nbuf, 1:nny, 1:Nbuf, 3)
  ar15_e(1:Nbuf, 1:nny, 1:Nbuf, 5  ) = Tgaz(1:Nbuf, 1:nny, 1:Nbuf   )

  ar25_e(1:Nbuf, 1:nny, 1:Nbuf, 1  ) = Eray(nnx+1-Nbuf:nnx, 1:nny, 1:Nbuf   )
  ar25_e(1:Nbuf, 1:nny, 1:Nbuf, 2  ) = Fray(nnx+1-Nbuf:nnx, 1:nny, 1:Nbuf, 1)
  ar25_e(1:Nbuf, 1:nny, 1:Nbuf, 3  ) = Fray(nnx+1-Nbuf:nnx, 1:nny, 1:Nbuf, 2)
  ar25_e(1:Nbuf, 1:nny, 1:Nbuf, 4  ) = Fray(nnx+1-Nbuf:nnx, 1:nny, 1:Nbuf, 3)
  ar25_e(1:Nbuf, 1:nny, 1:Nbuf, 5  ) = Tgaz(nnx+1-Nbuf:nnx, 1:nny, 1:Nbuf   )

  ar16_e(1:Nbuf, 1:nny, 1:Nbuf, 1  ) = Eray(1:Nbuf, 1:nny, nnz+1-Nbuf:nnz   )
  ar16_e(1:Nbuf, 1:nny, 1:Nbuf, 2  ) = Fray(1:Nbuf, 1:nny, nnz+1-Nbuf:nnz, 1)
  ar16_e(1:Nbuf, 1:nny, 1:Nbuf, 3  ) = Fray(1:Nbuf, 1:nny, nnz+1-Nbuf:nnz, 2)
  ar16_e(1:Nbuf, 1:nny, 1:Nbuf, 4  ) = Fray(1:Nbuf, 1:nny, nnz+1-Nbuf:nnz, 3)
  ar16_e(1:Nbuf, 1:nny, 1:Nbuf, 5  ) = Tgaz(1:Nbuf, 1:nny, nnz+1-Nbuf:nnz   )

  ar26_e(1:Nbuf, 1:nny, 1:Nbuf, 1  ) = Eray(nnx+1-Nbuf:nnx, 1:nny, nnz+1-Nbuf:nnz   )
  ar26_e(1:Nbuf, 1:nny, 1:Nbuf, 2  ) = Fray(nnx+1-Nbuf:nnx, 1:nny, nnz+1-Nbuf:nnz, 1)
  ar26_e(1:Nbuf, 1:nny, 1:Nbuf, 3  ) = Fray(nnx+1-Nbuf:nnx, 1:nny, nnz+1-Nbuf:nnz, 2)
  ar26_e(1:Nbuf, 1:nny, 1:Nbuf, 4  ) = Fray(nnx+1-Nbuf:nnx, 1:nny, nnz+1-Nbuf:nnz, 3)
  ar26_e(1:Nbuf, 1:nny, 1:Nbuf, 5  ) = Tgaz(nnx+1-Nbuf:nnx, 1:nny, nnz+1-Nbuf:nnz   )

  

  ar35_e(1:nnx, 1:Nbuf, 1:Nbuf, 1  ) = Eray(1:nnx, 1:Nbuf, 1:Nbuf   )
  ar35_e(1:nnx, 1:Nbuf, 1:Nbuf, 2  ) = Fray(1:nnx, 1:Nbuf, 1:Nbuf, 1)
  ar35_e(1:nnx, 1:Nbuf, 1:Nbuf, 3  ) = Fray(1:nnx, 1:Nbuf, 1:Nbuf, 2)
  ar35_e(1:nnx, 1:Nbuf, 1:Nbuf, 4  ) = Fray(1:nnx, 1:Nbuf, 1:Nbuf, 3)
  ar35_e(1:nnx, 1:Nbuf, 1:Nbuf, 5  ) = Tgaz(1:nnx, 1:Nbuf, 1:Nbuf   )

  ar45_e(1:nnx, 1:Nbuf, 1:Nbuf, 1  ) = Eray(1:nnx, nny+1-Nbuf:nny, 1:Nbuf   )
  ar45_e(1:nnx, 1:Nbuf, 1:Nbuf, 2  ) = Fray(1:nnx, nny+1-Nbuf:nny, 1:Nbuf, 1)
  ar45_e(1:nnx, 1:Nbuf, 1:Nbuf, 3  ) = Fray(1:nnx, nny+1-Nbuf:nny, 1:Nbuf, 2)
  ar45_e(1:nnx, 1:Nbuf, 1:Nbuf, 4  ) = Fray(1:nnx, nny+1-Nbuf:nny, 1:Nbuf, 3)
  ar45_e(1:nnx, 1:Nbuf, 1:Nbuf, 5  ) = Tgaz(1:nnx, nny+1-Nbuf:nny, 1:Nbuf   )

  ar36_e(1:nnx, 1:Nbuf, 1:Nbuf, 1  ) = Eray(1:nnx, 1:Nbuf, nnz+1-Nbuf:nnz   )
  ar36_e(1:nnx, 1:Nbuf, 1:Nbuf, 2  ) = Fray(1:nnx, 1:Nbuf, nnz+1-Nbuf:nnz, 1)
  ar36_e(1:nnx, 1:Nbuf, 1:Nbuf, 3  ) = Fray(1:nnx, 1:Nbuf, nnz+1-Nbuf:nnz, 2)
  ar36_e(1:nnx, 1:Nbuf, 1:Nbuf, 4  ) = Fray(1:nnx, 1:Nbuf, nnz+1-Nbuf:nnz, 3)
  ar36_e(1:nnx, 1:Nbuf, 1:Nbuf, 5  ) = Tgaz(1:nnx, 1:Nbuf, nnz+1-Nbuf:nnz   )

  ar46_e(1:nnx, 1:Nbuf, 1:Nbuf, 1  ) = Eray(1:nnx, nny+1-Nbuf:nny, nnz+1-Nbuf:nnz   )
  ar46_e(1:nnx, 1:Nbuf, 1:Nbuf, 2  ) = Fray(1:nnx, nny+1-Nbuf:nny, nnz+1-Nbuf:nnz, 1)
  ar46_e(1:nnx, 1:Nbuf, 1:Nbuf, 3  ) = Fray(1:nnx, nny+1-Nbuf:nny, nnz+1-Nbuf:nnz, 2)
  ar46_e(1:nnx, 1:Nbuf, 1:Nbuf, 4  ) = Fray(1:nnx, nny+1-Nbuf:nny, nnz+1-Nbuf:nnz, 3)
  ar46_e(1:nnx, 1:Nbuf, 1:Nbuf, 5  ) = Tgaz(1:nnx, nny+1-Nbuf:nny, nnz+1-Nbuf:nnz   )

 

  ! 8 corners
  coin135_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 1) = Eray(1:Nbuf, 1:Nbuf, 1:Nbuf   ) 
  coin135_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 2) = Fray(1:Nbuf, 1:Nbuf, 1:Nbuf, 1) 
  coin135_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 3) = Fray(1:Nbuf, 1:Nbuf, 1:Nbuf, 2) 
  coin135_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 4) = Fray(1:Nbuf, 1:Nbuf, 1:Nbuf, 3) 
  coin135_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 5) = Tgaz(1:Nbuf, 1:Nbuf, 1:Nbuf   ) 

  coin136_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 1) = Eray(1:Nbuf, 1:Nbuf, nnz+1-Nbuf:nnz   ) 
  coin136_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 2) = Fray(1:Nbuf, 1:Nbuf, nnz+1-Nbuf:nnz, 1) 
  coin136_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 3) = Fray(1:Nbuf, 1:Nbuf, nnz+1-Nbuf:nnz, 2) 
  coin136_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 4) = Fray(1:Nbuf, 1:Nbuf, nnz+1-Nbuf:nnz, 3) 
  coin136_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 5) = Tgaz(1:Nbuf, 1:Nbuf, nnz+1-Nbuf:nnz   ) 

  coin145_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 1) = Eray(1:Nbuf, nny+1-Nbuf:nny, 1:Nbuf   ) 
  coin145_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 2) = Fray(1:Nbuf, nny+1-Nbuf:nny, 1:Nbuf, 1) 
  coin145_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 3) = Fray(1:Nbuf, nny+1-Nbuf:nny, 1:Nbuf, 2) 
  coin145_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 4) = Fray(1:Nbuf, nny+1-Nbuf:nny, 1:Nbuf, 3) 
  coin145_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 5) = Tgaz(1:Nbuf, nny+1-Nbuf:nny, 1:Nbuf   ) 

  coin146_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 1) = Eray(1:Nbuf, nny+1-Nbuf:nny, nnz+1-Nbuf:nnz   ) 
  coin146_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 2) = Fray(1:Nbuf, nny+1-Nbuf:nny, nnz+1-Nbuf:nnz, 1) 
  coin146_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 3) = Fray(1:Nbuf, nny+1-Nbuf:nny, nnz+1-Nbuf:nnz, 2) 
  coin146_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 4) = Fray(1:Nbuf, nny+1-Nbuf:nny, nnz+1-Nbuf:nnz, 3) 
  coin146_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 5) = Tgaz(1:Nbuf, nny+1-Nbuf:nny, nnz+1-Nbuf:nnz   ) 


  coin235_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 1) = Eray(nnx+1-Nbuf:nnx, 1:Nbuf, 1:Nbuf   ) 
  coin235_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 2) = Fray(nnx+1-Nbuf:nnx, 1:Nbuf, 1:Nbuf, 1) 
  coin235_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 3) = Fray(nnx+1-Nbuf:nnx, 1:Nbuf, 1:Nbuf, 2) 
  coin235_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 4) = Fray(nnx+1-Nbuf:nnx, 1:Nbuf, 1:Nbuf, 3) 
  coin235_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 5) = Tgaz(nnx+1-Nbuf:nnx, 1:Nbuf, 1:Nbuf   ) 

  coin236_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 1) = Eray(nnx+1-Nbuf:nnx, 1:Nbuf, nnz+1-Nbuf:nnz   ) 
  coin236_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 2) = Fray(nnx+1-Nbuf:nnx, 1:Nbuf, nnz+1-Nbuf:nnz, 1) 
  coin236_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 3) = Fray(nnx+1-Nbuf:nnx, 1:Nbuf, nnz+1-Nbuf:nnz, 2) 
  coin236_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 4) = Fray(nnx+1-Nbuf:nnx, 1:Nbuf, nnz+1-Nbuf:nnz, 3) 
  coin236_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 5) = Tgaz(nnx+1-Nbuf:nnx, 1:Nbuf, nnz+1-Nbuf:nnz   ) 

  coin245_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 1) = Eray(nnx+1-Nbuf:nnx, nny+1-Nbuf:nny, 1:Nbuf   ) 
  coin245_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 2) = Fray(nnx+1-Nbuf:nnx, nny+1-Nbuf:nny, 1:Nbuf, 1) 
  coin245_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 3) = Fray(nnx+1-Nbuf:nnx, nny+1-Nbuf:nny, 1:Nbuf, 2) 
  coin245_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 4) = Fray(nnx+1-Nbuf:nnx, nny+1-Nbuf:nny, 1:Nbuf, 3) 
  coin245_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 5) = Tgaz(nnx+1-Nbuf:nnx, nny+1-Nbuf:nny, 1:Nbuf   ) 

  coin246_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 1) = Eray(nnx+1-Nbuf:nnx, nny+1-Nbuf:nny, nnz+1-Nbuf:nnz   ) 
  coin246_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 2) = Fray(nnx+1-Nbuf:nnx, nny+1-Nbuf:nny, nnz+1-Nbuf:nnz, 1) 
  coin246_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 3) = Fray(nnx+1-Nbuf:nnx, nny+1-Nbuf:nny, nnz+1-Nbuf:nnz, 2) 
  coin246_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 4) = Fray(nnx+1-Nbuf:nnx, nny+1-Nbuf:nny, nnz+1-Nbuf:nnz, 3) 
  coin246_e(1:Nbuf, 1:Nbuf, 1:Nbuf, 5) = Tgaz(nnx+1-Nbuf:nnx, nny+1-Nbuf:nny, nnz+1-Nbuf:nnz   ) 


  ! send data

  ! Faces
  destination = voisin(-1, 0, 0)
  call MPI_Isend(face1_e ,ic_f12, MPI_DOUBLE_PRECISION,destination,F1 ,MPI_Comm_world,Req( 1),ierr)

  destination = voisin(+1, 0, 0)
  call MPI_Isend(face2_e ,ic_f12, MPI_DOUBLE_PRECISION,destination,F2 ,MPI_COMM_WORLD,Req( 2),ierr)

  destination = voisin( 0,-1, 0)
  call MPI_Isend(face3_e ,ic_f34, MPI_DOUBLE_PRECISION,destination,F3 ,MPI_COMM_WORLD,Req( 3),ierr)

  destination = voisin( 0,+1, 0)
  call MPI_Isend(face4_e ,ic_f34, MPI_DOUBLE_PRECISION,destination,F4 ,MPI_COMM_WORLD,Req( 4),ierr)

  destination = voisin( 0, 0,-1)
  call MPI_Isend(face5_e ,ic_f56, MPI_DOUBLE_PRECISION,destination,F5 ,MPI_COMM_WORLD,Req( 5),ierr)

  destination = voisin( 0, 0,+1)
  call MPI_Isend(face6_e ,ic_f56, MPI_DOUBLE_PRECISION,destination,F6 ,MPI_COMM_WORLD,Req( 6),ierr)



  ! Edges
  destination = voisin(-1,-1,0)
  call MPI_Isend(ar13_e,ic_arz, MPI_DOUBLE_PRECISION,destination,AR13   ,MPI_COMM_WORLD,Req( 7),ierr)

  destination = voisin(-1,+1,0)
  call MPI_Isend(ar14_e,ic_arz, MPI_DOUBLE_PRECISION,destination,AR14   ,MPI_COMM_WORLD,Req( 8),ierr)

  destination = voisin(+1,-1,0)
  call MPI_Isend(ar23_e,ic_arz, MPI_DOUBLE_PRECISION,destination,AR23   ,MPI_COMM_WORLD,Req( 9),ierr)

  destination = voisin(+1,+1,0)
  call MPI_Isend(ar24_e,ic_arz, MPI_DOUBLE_PRECISION,destination,AR24   ,MPI_COMM_WORLD,Req(10),ierr)


  destination = voisin(-1, 0,-1)
  call MPI_Isend(ar15_e,ic_ary, MPI_DOUBLE_PRECISION,destination,AR15   ,MPI_COMM_WORLD,Req(11),ierr)

  destination = voisin(-1, 0,+1)
  call MPI_Isend(ar16_e,ic_ary, MPI_DOUBLE_PRECISION,destination,AR16   ,MPI_COMM_WORLD,Req(12),ierr)

  destination = voisin(+1, 0,-1)
  call MPI_Isend(ar25_e,ic_ary, MPI_DOUBLE_PRECISION,destination,AR25   ,MPI_COMM_WORLD,Req(13),ierr)

  destination = voisin(+1, 0,+1)
  call MPI_Isend(ar26_e,ic_ary, MPI_DOUBLE_PRECISION,destination,AR26   ,MPI_COMM_WORLD,Req(14),ierr)


  destination = voisin( 0,-1,-1)
  call MPI_Isend(ar35_e,ic_arx, MPI_DOUBLE_PRECISION,destination,AR35   ,MPI_COMM_WORLD,Req(15),ierr)

  destination = voisin( 0,-1,+1)
  call MPI_Isend(ar36_e,ic_arx, MPI_DOUBLE_PRECISION,destination,AR36   ,MPI_COMM_WORLD,Req(16),ierr)

  destination = voisin( 0,+1,-1)
  call MPI_Isend(ar45_e,ic_arx, MPI_DOUBLE_PRECISION,destination,AR45   ,MPI_COMM_WORLD,Req(17),ierr)

  destination = voisin( 0,+1,+1)
  call MPI_Isend(ar46_e,ic_arx, MPI_DOUBLE_PRECISION,destination,AR46   ,MPI_COMM_WORLD,Req(18),ierr)


  ! Corners
  destination = voisin( -1,-1,-1)
  call MPI_Isend(coin135_e,ic_coin, MPI_DOUBLE_PRECISION,destination,C135   ,MPI_COMM_WORLD,Req(19),ierr)

  destination = voisin( -1,-1,+1)
  call MPI_Isend(coin136_e,ic_coin, MPI_DOUBLE_PRECISION,destination,C136   ,MPI_COMM_WORLD,Req(20),ierr)

  destination = voisin( -1,+1,-1)
  call MPI_Isend(coin145_e,ic_coin, MPI_DOUBLE_PRECISION,destination,C145   ,MPI_COMM_WORLD,Req(21),ierr)

  destination = voisin( -1,+1,+1)
  call MPI_Isend(coin146_e,ic_coin, MPI_DOUBLE_PRECISION,destination,C146   ,MPI_COMM_WORLD,Req(22),ierr)

  destination = voisin( +1,-1,-1)
  call MPI_Isend(coin235_e,ic_coin, MPI_DOUBLE_PRECISION,destination,C235   ,MPI_COMM_WORLD,Req(23),ierr)

  destination = voisin( +1,-1,+1)
  call MPI_Isend(coin236_e,ic_coin, MPI_DOUBLE_PRECISION,destination,C236   ,MPI_COMM_WORLD,Req(24),ierr)

  destination = voisin( +1,+1,-1)
  call MPI_Isend(coin245_e,ic_coin, MPI_DOUBLE_PRECISION,destination,C245   ,MPI_COMM_WORLD,Req(25),ierr)

  destination = voisin( +1,+1,+1)
  call MPI_Isend(coin246_e,ic_coin, MPI_DOUBLE_PRECISION,destination,C246   ,MPI_COMM_WORLD,Req(26),ierr)

 

  ! receive data

  ! Faces
  emetteur = voisin(+1, 0, 0)
  call MPI_Irecv(face2_r ,ic_f12, MPI_DOUBLE_PRECISION,emetteur,F1 ,MPI_Comm_world,Req(27),ierr)

  emetteur = voisin(-1, 0, 0)
  call MPI_Irecv(face1_r ,ic_f12, MPI_DOUBLE_PRECISION,emetteur,F2 ,MPI_COMM_WORLD,Req(28),ierr)

  emetteur = voisin( 0,+1, 0)
  call MPI_Irecv(face4_r ,ic_f34, MPI_DOUBLE_PRECISION,emetteur,F3 ,MPI_COMM_WORLD,Req(29),ierr)

  emetteur = voisin( 0,-1, 0)
  call MPI_Irecv(face3_r ,ic_f34, MPI_DOUBLE_PRECISION,emetteur,F4 ,MPI_COMM_WORLD,Req(30),ierr)

  emetteur = voisin( 0, 0,+1)
  call MPI_Irecv(face6_r ,ic_f56, MPI_DOUBLE_PRECISION,emetteur,F5 ,MPI_COMM_WORLD,Req(31),ierr)

  emetteur = voisin( 0, 0,-1)
  call MPI_Irecv(face5_r ,ic_f56, MPI_DOUBLE_PRECISION,emetteur,F6 ,MPI_COMM_WORLD,Req(32),ierr)



  ! Edges
  emetteur = voisin(+1,+1,0)
  call MPI_Irecv(ar24_r,ic_arz, MPI_DOUBLE_PRECISION,emetteur,AR13   ,MPI_COMM_WORLD,Req(33),ierr)

  emetteur = voisin(+1,-1,0)
  call MPI_Irecv(ar23_r,ic_arz, MPI_DOUBLE_PRECISION,emetteur,AR14   ,MPI_COMM_WORLD,Req(34),ierr)

  emetteur = voisin(-1,+1,0)
  call MPI_Irecv(ar14_r,ic_arz, MPI_DOUBLE_PRECISION,emetteur,AR23   ,MPI_COMM_WORLD,Req(35),ierr)

  emetteur = voisin(-1,-1,0)
  call MPI_Irecv(ar13_r,ic_arz, MPI_DOUBLE_PRECISION,emetteur,AR24   ,MPI_COMM_WORLD,Req(36),ierr)


  emetteur = voisin(+1, 0,+1)
  call MPI_Irecv(ar26_r,ic_ary, MPI_DOUBLE_PRECISION,emetteur,AR15   ,MPI_COMM_WORLD,Req(37),ierr)

  emetteur = voisin(+1, 0,-1)
  call MPI_Irecv(ar25_r,ic_ary, MPI_DOUBLE_PRECISION,emetteur,AR16   ,MPI_COMM_WORLD,Req(38),ierr)

  emetteur = voisin(-1, 0,+1)
  call MPI_Irecv(ar16_r,ic_ary, MPI_DOUBLE_PRECISION,emetteur,AR25   ,MPI_COMM_WORLD,Req(39),ierr)

  emetteur = voisin(-1, 0,-1)
  call MPI_Irecv(ar15_r,ic_ary, MPI_DOUBLE_PRECISION,emetteur,AR26   ,MPI_COMM_WORLD,Req(40),ierr)


  emetteur = voisin( 0,+1,+1)
  call MPI_Irecv(ar46_r,ic_arx, MPI_DOUBLE_PRECISION,emetteur,AR35   ,MPI_COMM_WORLD,Req(41),ierr)

  emetteur = voisin( 0,+1,-1)
  call MPI_Irecv(ar45_r,ic_arx, MPI_DOUBLE_PRECISION,emetteur,AR36   ,MPI_COMM_WORLD,Req(42),ierr)

  emetteur = voisin( 0,-1,+1)
  call MPI_Irecv(ar36_r,ic_arx, MPI_DOUBLE_PRECISION,emetteur,AR45   ,MPI_COMM_WORLD,Req(43),ierr)

  emetteur = voisin( 0,-1,-1)
  call MPI_Irecv(ar35_r,ic_arx, MPI_DOUBLE_PRECISION,emetteur,AR46   ,MPI_COMM_WORLD,Req(44),ierr)


  ! Corners
  emetteur = voisin( +1,+1,+1)
  call MPI_Irecv(coin246_r,ic_coin, MPI_DOUBLE_PRECISION,emetteur,C135   ,MPI_COMM_WORLD,Req(45),ierr)

  emetteur = voisin( +1,+1,-1)
  call MPI_Irecv(coin245_r,ic_coin, MPI_DOUBLE_PRECISION,emetteur,C136   ,MPI_COMM_WORLD,Req(46),ierr)

  emetteur = voisin( +1,-1,+1)
  call MPI_Irecv(coin236_r,ic_coin, MPI_DOUBLE_PRECISION,emetteur,C145   ,MPI_COMM_WORLD,Req(47),ierr)

  emetteur = voisin( +1,-1,-1)
  call MPI_Irecv(coin235_r,ic_coin, MPI_DOUBLE_PRECISION,emetteur,C146   ,MPI_COMM_WORLD,Req(48),ierr)

  emetteur = voisin( -1,+1,+1)
  call MPI_Irecv(coin146_r,ic_coin, MPI_DOUBLE_PRECISION,emetteur,C235   ,MPI_COMM_WORLD,Req(49),ierr)

  emetteur = voisin( -1,+1,-1)
  call MPI_Irecv(coin145_r,ic_coin, MPI_DOUBLE_PRECISION,emetteur,C236   ,MPI_COMM_WORLD,Req(50),ierr)

  emetteur = voisin( -1,-1,+1)
  call MPI_Irecv(coin136_r,ic_coin, MPI_DOUBLE_PRECISION,emetteur,C245   ,MPI_COMM_WORLD,Req(51),ierr)

  emetteur = voisin( -1,-1,-1)
  call MPI_Irecv(coin135_r,ic_coin, MPI_DOUBLE_PRECISION,emetteur,C246   ,MPI_COMM_WORLD,Req(52),ierr)

  call MPI_Waitall(52,Req,Stat,ierr)

  call MPI_BARRIER(MPI_COMM_WORLD,ierr)


  ! 6 faces
  Eray(1-Nbuf:0,1:nny,1:nnz  ) = face1_r(1:Nbuf,1:nny,1:nnz,1) 
  Fray(1-Nbuf:0,1:nny,1:nnz,1) = face1_r(1:Nbuf,1:nny,1:nnz,2)  
  Fray(1-Nbuf:0,1:nny,1:nnz,2) = face1_r(1:Nbuf,1:nny,1:nnz,3) 
  Fray(1-Nbuf:0,1:nny,1:nnz,3) = face1_r(1:Nbuf,1:nny,1:nnz,4)
  Tgaz(1-Nbuf:0,1:nny,1:nnz  ) = face1_r(1:Nbuf,1:nny,1:nnz,5) 

  Eray(nnx+1:nnx+Nbuf,1:nny,1:nnz  ) = face2_r(1:Nbuf,1:nny,1:nnz,1) 
  Fray(nnx+1:nnx+Nbuf,1:nny,1:nnz,1) = face2_r(1:Nbuf,1:nny,1:nnz,2) 
  Fray(nnx+1:nnx+Nbuf,1:nny,1:nnz,2) = face2_r(1:Nbuf,1:nny,1:nnz,3) 
  Fray(nnx+1:nnx+Nbuf,1:nny,1:nnz,3) = face2_r(1:Nbuf,1:nny,1:nnz,4) 
  Tgaz(nnx+1:nnx+Nbuf,1:nny,1:nnz  ) = face2_r(1:Nbuf,1:nny,1:nnz,5) 

  Eray(1:nnx,1-Nbuf:0,1:nnz  ) = face3_r(1:nnx,1:Nbuf,1:nnz,1)  
  Fray(1:nnx,1-Nbuf:0,1:nnz,1) = face3_r(1:nnx,1:Nbuf,1:nnz,2)  
  Fray(1:nnx,1-Nbuf:0,1:nnz,2) = face3_r(1:nnx,1:Nbuf,1:nnz,3)  
  Fray(1:nnx,1-Nbuf:0,1:nnz,3) = face3_r(1:nnx,1:Nbuf,1:nnz,4)  
  Tgaz(1:nnx,1-Nbuf:0,1:nnz  ) = face3_r(1:nnx,1:Nbuf,1:nnz,5)  

  Eray(1:nnx,nny+1:nny+Nbuf,1:nnz  ) = face4_r(1:nnx,1:Nbuf,1:nnz,1) 
  Fray(1:nnx,nny+1:nny+Nbuf,1:nnz,1) = face4_r(1:nnx,1:Nbuf,1:nnz,2) 
  Fray(1:nnx,nny+1:nny+Nbuf,1:nnz,2) = face4_r(1:nnx,1:Nbuf,1:nnz,3) 
  Fray(1:nnx,nny+1:nny+Nbuf,1:nnz,3) = face4_r(1:nnx,1:Nbuf,1:nnz,4) 
  Tgaz(1:nnx,nny+1:nny+Nbuf,1:nnz  ) = face4_r(1:nnx,1:Nbuf,1:nnz,5) 

  Eray(1:nnx,1:nny,1-Nbuf:0  ) = face5_r(1:nnx,1:nny,1:Nbuf,1) 
  Fray(1:nnx,1:nny,1-Nbuf:0,1) = face5_r(1:nnx,1:nny,1:Nbuf,2) 
  Fray(1:nnx,1:nny,1-Nbuf:0,2) = face5_r(1:nnx,1:nny,1:Nbuf,3) 
  Fray(1:nnx,1:nny,1-Nbuf:0,3) = face5_r(1:nnx,1:nny,1:Nbuf,4) 
  Tgaz(1:nnx,1:nny,1-Nbuf:0  ) = face5_r(1:nnx,1:nny,1:Nbuf,5) 

  Eray(1:nnx,1:nny,nnz+1:nnz+Nbuf  ) = face6_r(1:nnx,1:nny,1:Nbuf,1) 
  Fray(1:nnx,1:nny,nnz+1:nnz+Nbuf,1) = face6_r(1:nnx,1:nny,1:Nbuf,2) 
  Fray(1:nnx,1:nny,nnz+1:nnz+Nbuf,2) = face6_r(1:nnx,1:nny,1:Nbuf,3) 
  Fray(1:nnx,1:nny,nnz+1:nnz+Nbuf,3) = face6_r(1:nnx,1:nny,1:Nbuf,4) 
  Tgaz(1:nnx,1:nny,nnz+1:nnz+Nbuf  ) = face6_r(1:nnx,1:nny,1:Nbuf,5) 
  
  ! 12 edges
  Eray(1-Nbuf:0, 1-Nbuf:0, 1:nnz   ) = ar13_r(1:Nbuf, 1:Nbuf, 1:nnz, 1  )
  Fray(1-Nbuf:0, 1-Nbuf:0, 1:nnz, 1) = ar13_r(1:Nbuf, 1:Nbuf, 1:nnz, 2  )
  Fray(1-Nbuf:0, 1-Nbuf:0, 1:nnz, 2) = ar13_r(1:Nbuf, 1:Nbuf, 1:nnz, 3  )
  Fray(1-Nbuf:0, 1-Nbuf:0, 1:nnz, 3) = ar13_r(1:Nbuf, 1:Nbuf, 1:nnz, 4  )
  Tgaz(1-Nbuf:0, 1-Nbuf:0, 1:nnz   ) = ar13_r(1:Nbuf, 1:Nbuf, 1:nnz, 5  )

  Eray(1-Nbuf:0, nny+1:nny+Nbuf, 1:nnz   ) = ar14_r(1:Nbuf, 1:Nbuf, 1:nnz, 1  )
  Fray(1-Nbuf:0, nny+1:nny+Nbuf, 1:nnz, 1) = ar14_r(1:Nbuf, 1:Nbuf, 1:nnz, 2  )
  Fray(1-Nbuf:0, nny+1:nny+Nbuf, 1:nnz, 2) = ar14_r(1:Nbuf, 1:Nbuf, 1:nnz, 3  )
  Fray(1-Nbuf:0, nny+1:nny+Nbuf, 1:nnz, 3) = ar14_r(1:Nbuf, 1:Nbuf, 1:nnz, 4  )
  Tgaz(1-Nbuf:0, nny+1:nny+Nbuf, 1:nnz   ) = ar14_r(1:Nbuf, 1:Nbuf, 1:nnz, 5  )

  Eray(nnx+1:nnx+Nbuf, 1-Nbuf:0, 1:nnz   ) = ar23_r(1:Nbuf, 1:Nbuf, 1:nnz, 1  )
  Fray(nnx+1:nnx+Nbuf, 1-Nbuf:0, 1:nnz, 1) = ar23_r(1:Nbuf, 1:Nbuf, 1:nnz, 2  )
  Fray(nnx+1:nnx+Nbuf, 1-Nbuf:0, 1:nnz, 2) = ar23_r(1:Nbuf, 1:Nbuf, 1:nnz, 3  )
  Fray(nnx+1:nnx+Nbuf, 1-Nbuf:0, 1:nnz, 3) = ar23_r(1:Nbuf, 1:Nbuf, 1:nnz, 4  )
  Tgaz(nnx+1:nnx+Nbuf, 1-Nbuf:0, 1:nnz   ) = ar23_r(1:Nbuf, 1:Nbuf, 1:nnz, 5  )

  Eray(nnx+1:nnx+Nbuf, nny+1:nny+Nbuf, 1:nnz   ) = ar24_r(1:Nbuf, 1:Nbuf, 1:nnz, 1  )
  Fray(nnx+1:nnx+Nbuf, nny+1:nny+Nbuf, 1:nnz, 1) = ar24_r(1:Nbuf, 1:Nbuf, 1:nnz, 2  )
  Fray(nnx+1:nnx+Nbuf, nny+1:nny+Nbuf, 1:nnz, 2) = ar24_r(1:Nbuf, 1:Nbuf, 1:nnz, 3  )
  Fray(nnx+1:nnx+Nbuf, nny+1:nny+Nbuf, 1:nnz, 3) = ar24_r(1:Nbuf, 1:Nbuf, 1:nnz, 4  )
  Tgaz(nnx+1:nnx+Nbuf, nny+1:nny+Nbuf, 1:nnz   ) = ar24_r(1:Nbuf, 1:Nbuf, 1:nnz, 5  )


  Eray(1-Nbuf:0, 1:nny, 1-Nbuf:0   ) = ar15_r(1:Nbuf, 1:nny, 1:Nbuf, 1  ) 
  Fray(1-Nbuf:0, 1:nny, 1-Nbuf:0, 1) = ar15_r(1:Nbuf, 1:nny, 1:Nbuf, 2  ) 
  Fray(1-Nbuf:0, 1:nny, 1-Nbuf:0, 2) = ar15_r(1:Nbuf, 1:nny, 1:Nbuf, 3  ) 
  Fray(1-Nbuf:0, 1:nny, 1-Nbuf:0, 3) = ar15_r(1:Nbuf, 1:nny, 1:Nbuf, 4  ) 
  Tgaz(1-Nbuf:0, 1:nny, 1-Nbuf:0   ) = ar15_r(1:Nbuf, 1:nny, 1:Nbuf, 5  ) 

  Eray(nnx+1:nnx+Nbuf, 1:nny, 1-Nbuf:0   ) = ar25_r(1:Nbuf, 1:nny, 1:Nbuf, 1  )
  Fray(nnx+1:nnx+Nbuf, 1:nny, 1-Nbuf:0, 1) = ar25_r(1:Nbuf, 1:nny, 1:Nbuf, 2  )
  Fray(nnx+1:nnx+Nbuf, 1:nny, 1-Nbuf:0, 2) = ar25_r(1:Nbuf, 1:nny, 1:Nbuf, 3  )
  Fray(nnx+1:nnx+Nbuf, 1:nny, 1-Nbuf:0, 3) = ar25_r(1:Nbuf, 1:nny, 1:Nbuf, 4  )
  Tgaz(nnx+1:nnx+Nbuf, 1:nny, 1-Nbuf:0   ) = ar25_r(1:Nbuf, 1:nny, 1:Nbuf, 5  )

  Eray(1-Nbuf:0, 1:nny, nnz+1:nnz+Nbuf   ) = ar16_r(1:Nbuf, 1:nny, 1:Nbuf, 1  )
  Fray(1-Nbuf:0, 1:nny, nnz+1:nnz+Nbuf, 1) = ar16_r(1:Nbuf, 1:nny, 1:Nbuf, 2  )
  Fray(1-Nbuf:0, 1:nny, nnz+1:nnz+Nbuf, 2) = ar16_r(1:Nbuf, 1:nny, 1:Nbuf, 3  )
  Fray(1-Nbuf:0, 1:nny, nnz+1:nnz+Nbuf, 3) = ar16_r(1:Nbuf, 1:nny, 1:Nbuf, 4  )
  Tgaz(1-Nbuf:0, 1:nny, nnz+1:nnz+Nbuf   ) = ar16_r(1:Nbuf, 1:nny, 1:Nbuf, 5  )

  Eray(nnx+1:nnx+Nbuf, 1:nny, nnz+1:nnz+Nbuf   ) = ar26_r(1:Nbuf, 1:nny, 1:Nbuf, 1  )
  Fray(nnx+1:nnx+Nbuf, 1:nny, nnz+1:nnz+Nbuf, 1) = ar26_r(1:Nbuf, 1:nny, 1:Nbuf, 2  )
  Fray(nnx+1:nnx+Nbuf, 1:nny, nnz+1:nnz+Nbuf, 2) = ar26_r(1:Nbuf, 1:nny, 1:Nbuf, 3  )
  Fray(nnx+1:nnx+Nbuf, 1:nny, nnz+1:nnz+Nbuf, 3) = ar26_r(1:Nbuf, 1:nny, 1:Nbuf, 4  )
  Tgaz(nnx+1:nnx+Nbuf, 1:nny, nnz+1:nnz+Nbuf   ) = ar26_r(1:Nbuf, 1:nny, 1:Nbuf, 5  )
  

  Eray(1:nnx, 1-Nbuf:0, 1-Nbuf:0   ) = ar35_r(1:nnx, 1:Nbuf, 1:Nbuf, 1  )
  Fray(1:nnx, 1-Nbuf:0, 1-Nbuf:0, 1) = ar35_r(1:nnx, 1:Nbuf, 1:Nbuf, 2  )
  Fray(1:nnx, 1-Nbuf:0, 1-Nbuf:0, 2) = ar35_r(1:nnx, 1:Nbuf, 1:Nbuf, 3  )
  Fray(1:nnx, 1-Nbuf:0, 1-Nbuf:0, 3) = ar35_r(1:nnx, 1:Nbuf, 1:Nbuf, 4  )
  Tgaz(1:nnx, 1-Nbuf:0, 1-Nbuf:0   ) = ar35_r(1:nnx, 1:Nbuf, 1:Nbuf, 5  )

  Eray(1:nnx, nny+1:nny+Nbuf, 1-Nbuf:0   ) = ar45_r(1:nnx, 1:Nbuf, 1:Nbuf, 1  ) 
  Fray(1:nnx, nny+1:nny+Nbuf, 1-Nbuf:0, 1) = ar45_r(1:nnx, 1:Nbuf, 1:Nbuf, 2  ) 
  Fray(1:nnx, nny+1:nny+Nbuf, 1-Nbuf:0, 2) = ar45_r(1:nnx, 1:Nbuf, 1:Nbuf, 3  ) 
  Fray(1:nnx, nny+1:nny+Nbuf, 1-Nbuf:0, 3) = ar45_r(1:nnx, 1:Nbuf, 1:Nbuf, 4  ) 
  Tgaz(1:nnx, nny+1:nny+Nbuf, 1-Nbuf:0   ) = ar45_r(1:nnx, 1:Nbuf, 1:Nbuf, 5  ) 

  Eray(1:nnx, 1-Nbuf:0, nnz+1:nnz+Nbuf   ) = ar36_r(1:nnx, 1:Nbuf, 1:Nbuf, 1  ) 
  Fray(1:nnx, 1-Nbuf:0, nnz+1:nnz+Nbuf, 1) = ar36_r(1:nnx, 1:Nbuf, 1:Nbuf, 2  ) 
  Fray(1:nnx, 1-Nbuf:0, nnz+1:nnz+Nbuf, 2) = ar36_r(1:nnx, 1:Nbuf, 1:Nbuf, 3  ) 
  Fray(1:nnx, 1-Nbuf:0, nnz+1:nnz+Nbuf, 3) = ar36_r(1:nnx, 1:Nbuf, 1:Nbuf, 4  ) 
  Tgaz(1:nnx, 1-Nbuf:0, nnz+1:nnz+Nbuf   ) = ar36_r(1:nnx, 1:Nbuf, 1:Nbuf, 5  ) 

  Eray(1:nnx, nny+1:nny+Nbuf, nnz+1:nnz+Nbuf   ) = ar46_r(1:nnx, 1:Nbuf, 1:Nbuf, 1  )
  Fray(1:nnx, nny+1:nny+Nbuf, nnz+1:nnz+Nbuf, 1) = ar46_r(1:nnx, 1:Nbuf, 1:Nbuf, 2  )
  Fray(1:nnx, nny+1:nny+Nbuf, nnz+1:nnz+Nbuf, 2) = ar46_r(1:nnx, 1:Nbuf, 1:Nbuf, 3  )
  Fray(1:nnx, nny+1:nny+Nbuf, nnz+1:nnz+Nbuf, 3) = ar46_r(1:nnx, 1:Nbuf, 1:Nbuf, 4  )
  Tgaz(1:nnx, nny+1:nny+Nbuf, nnz+1:nnz+Nbuf   ) = ar46_r(1:nnx, 1:Nbuf, 1:Nbuf, 5  )

 

  ! 8 corners
  Eray(1-Nbuf:0, 1-Nbuf:0, 1-Nbuf:0   ) = coin135_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 1)  
  Fray(1-Nbuf:0, 1-Nbuf:0, 1-Nbuf:0, 1) = coin135_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 2)  
  Fray(1-Nbuf:0, 1-Nbuf:0, 1-Nbuf:0, 2) = coin135_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 3)  
  Fray(1-Nbuf:0, 1-Nbuf:0, 1-Nbuf:0, 3) = coin135_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 4)  
  Tgaz(1-Nbuf:0, 1-Nbuf:0, 1-Nbuf:0   ) = coin135_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 5)  

  Eray(1-Nbuf:0, 1-Nbuf:0, nnz+1:nnz+Nbuf   ) = coin136_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 1)
  Fray(1-Nbuf:0, 1-Nbuf:0, nnz+1:nnz+Nbuf, 1) = coin136_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 2)
  Fray(1-Nbuf:0, 1-Nbuf:0, nnz+1:nnz+Nbuf, 2) = coin136_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 3)
  Fray(1-Nbuf:0, 1-Nbuf:0, nnz+1:nnz+Nbuf, 3) = coin136_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 4)
  Tgaz(1-Nbuf:0, 1-Nbuf:0, nnz+1:nnz+Nbuf   ) = coin136_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 5)

  Eray(1-Nbuf:0, nny+1:nny+Nbuf, 1-Nbuf:0   ) = coin145_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 1)  
  Fray(1-Nbuf:0, nny+1:nny+Nbuf, 1-Nbuf:0, 1) = coin145_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 2)  
  Fray(1-Nbuf:0, nny+1:nny+Nbuf, 1-Nbuf:0, 2) = coin145_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 3)  
  Fray(1-Nbuf:0, nny+1:nny+Nbuf, 1-Nbuf:0, 3) = coin145_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 4)  
  Tgaz(1-Nbuf:0, nny+1:nny+Nbuf, 1-Nbuf:0   ) = coin145_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 5)  

  Eray(1-Nbuf:0, nny+1:nny+Nbuf, nnz+1:nnz+Nbuf   ) = coin146_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 1) 
  Fray(1-Nbuf:0, nny+1:nny+Nbuf, nnz+1:nnz+Nbuf, 1) = coin146_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 2) 
  Fray(1-Nbuf:0, nny+1:nny+Nbuf, nnz+1:nnz+Nbuf, 2) = coin146_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 3) 
  Fray(1-Nbuf:0, nny+1:nny+Nbuf, nnz+1:nnz+Nbuf, 3) = coin146_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 4) 
  Tgaz(1-Nbuf:0, nny+1:nny+Nbuf, nnz+1:nnz+Nbuf   ) = coin146_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 5) 


  Eray(nnx+1:nnx+Nbuf, 1-Nbuf:0, 1-Nbuf:0   ) = coin235_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 1) 
  Fray(nnx+1:nnx+Nbuf, 1-Nbuf:0, 1-Nbuf:0, 1) = coin235_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 2) 
  Fray(nnx+1:nnx+Nbuf, 1-Nbuf:0, 1-Nbuf:0, 2) = coin235_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 3) 
  Fray(nnx+1:nnx+Nbuf, 1-Nbuf:0, 1-Nbuf:0, 3) = coin235_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 4) 
  Tgaz(nnx+1:nnx+Nbuf, 1-Nbuf:0, 1-Nbuf:0   ) = coin235_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 5) 

  Eray(nnx+1:nnx+Nbuf, 1-Nbuf:0, nnz+1:nnz+Nbuf   ) = coin236_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 1) 
  Fray(nnx+1:nnx+Nbuf, 1-Nbuf:0, nnz+1:nnz+Nbuf, 1) = coin236_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 2) 
  Fray(nnx+1:nnx+Nbuf, 1-Nbuf:0, nnz+1:nnz+Nbuf, 2) = coin236_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 3) 
  Fray(nnx+1:nnx+Nbuf, 1-Nbuf:0, nnz+1:nnz+Nbuf, 3) = coin236_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 4) 
  Tgaz(nnx+1:nnx+Nbuf, 1-Nbuf:0, nnz+1:nnz+Nbuf   ) = coin236_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 5)  

  Eray(nnx+1:nnx+Nbuf, nny+1:nny+Nbuf, 1-Nbuf:0   ) = coin245_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 1) 
  Fray(nnx+1:nnx+Nbuf, nny+1:nny+Nbuf, 1-Nbuf:0, 1) = coin245_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 2) 
  Fray(nnx+1:nnx+Nbuf, nny+1:nny+Nbuf, 1-Nbuf:0, 2) = coin245_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 3) 
  Fray(nnx+1:nnx+Nbuf, nny+1:nny+Nbuf, 1-Nbuf:0, 3) = coin245_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 4) 
  Tgaz(nnx+1:nnx+Nbuf, nny+1:nny+Nbuf, 1-Nbuf:0   ) = coin245_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 5) 

  Eray(nnx+1:nnx+Nbuf, nny+1:nny+Nbuf, nnz+1:nnz+Nbuf   ) = coin246_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 1)
  Fray(nnx+1:nnx+Nbuf, nny+1:nny+Nbuf, nnz+1:nnz+Nbuf, 1) = coin246_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 2)
  Fray(nnx+1:nnx+Nbuf, nny+1:nny+Nbuf, nnz+1:nnz+Nbuf, 2) = coin246_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 3)
  Fray(nnx+1:nnx+Nbuf, nny+1:nny+Nbuf, nnz+1:nnz+Nbuf, 3) = coin246_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 4)
  Tgaz(nnx+1:nnx+Nbuf, nny+1:nny+Nbuf, nnz+1:nnz+Nbuf   ) = coin246_r(1:Nbuf, 1:Nbuf, 1:Nbuf, 5)
  
  if(verbose) write(*,*) 'End of com_3D_ray'
  
  return
  
end subroutine com_3D_ray

#endif
