!> \file
!! Contains functions source_ray() and derive_source_ray()
!<

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Function SOURCE_RAY
!
!> Computes the energy-matter coupling source term
!! \f$ a_{R} T^{4} \f$ for the M1 radiative transfer.
!<
function source_ray(Temp)

  use unites, only : a_r

  implicit none

  real :: source_ray,Temp

  source_ray = a_r*Temp**4

end function source_ray

!###########################################################
!###########################################################
!###########################################################
!###########################################################

!  Function DERIVE_SOURCE_RAY
!
!> Computes the derivative of the energy-matter coupling
!! source term \f$ 4 a_{R} T^{3} \f$ for the M1 radiative
!! transfer.
!<
function derive_source_ray(Temp)

  use unites, only : a_r

  implicit none

  real :: derive_source_ray,Temp

  derive_source_ray = 4.*a_r*Temp**3

end function derive_source_ray
