!     ---------------
subroutine INIT
!     ---------------
  use parameters, only:nx,ndim,nx_glob,nbuf
  use unites
  use divers
  use divers_ray , only : rho1,T1,ordre_des_faces_pour_le_ray
  use var
  use varray
  use geom  , only : x,x_glob,Lbox,box_min,box_max
  use cl    , only : icl, rho_l, rhou_l, E_l, T_l
  use para  , only : mype,x_start

#if RAYO==1
  use cl_ray, only : Fr_l, Er_l, Tr_l
#endif

#if DIF==1
  use diffusion
  use cl_dif
#endif

  implicit none

  integer :: i,idim,i1,i2,nx_dim,idir

  real ,dimension(1:3) :: dx

  real :: Tray1 !rho1 and T1 are already included in module divers
  real :: u1,E1
  real :: T_s,Tr_s

  namelist/condinit/rho1,u1,T1,Tray1,Lbox,mu,box_min,idir
!----------------------

  if(verbose) write(*,*) 'Begin init'

! Begin time
  temps = 0.

  mu     =  1.
  Tray1  =  1.

  Lbox = 1.
  box_min = 0.

  idir=1

  ! Reading the namelist (previous values are erased)
  if(mype == 0) write(*,'(a)') 'reading namelist CONDINIT'
  open  (1,file='liste.nml')
  rewind(1                 )
  read  (1,NML=condinit    )
  close (1                 )

  tend   = tend   * seconde
  dt_max = dt_max * seconde

  ! Order of the faces for the radiative boundary conditions
  select case (idir)
  case (1)
     ordre_des_faces_pour_le_ray(1) = 1
     ordre_des_faces_pour_le_ray(2) = 2
     ordre_des_faces_pour_le_ray(3) = 3
     ordre_des_faces_pour_le_ray(4) = 4
     ordre_des_faces_pour_le_ray(5) = 5
     ordre_des_faces_pour_le_ray(6) = 6
  case (2)
     ordre_des_faces_pour_le_ray(1) = 3
     ordre_des_faces_pour_le_ray(2) = 4
     ordre_des_faces_pour_le_ray(3) = 1
     ordre_des_faces_pour_le_ray(4) = 2
     ordre_des_faces_pour_le_ray(5) = 5
     ordre_des_faces_pour_le_ray(6) = 6
  case (3)
     ordre_des_faces_pour_le_ray(1) = 5
     ordre_des_faces_pour_le_ray(2) = 6
     ordre_des_faces_pour_le_ray(3) = 1
     ordre_des_faces_pour_le_ray(4) = 2
     ordre_des_faces_pour_le_ray(5) = 3
     ordre_des_faces_pour_le_ray(6) = 4
  end select

!  Mesh
  Lbox(1:ndim) =  Lbox(1:ndim) * centimetre 

  if( mype == 0 ) then
     write(*,*)
     write(*,*) 'Dimensions of the global simulation (in cells numbers and centimetres):'
     write(*,001) ndim
     write(*,002) nx_glob(1),nx_glob(2),nx_glob(3),Lbox(1)/centimetre,Lbox(2)/centimetre,Lbox(3)/centimetre
     write(*,*)
  endif

001 format('ndim = ',I1)
002 format('nx=',I4,1x,'ny=',I4,1x,'nz=',I4,1x,'Lx=',1pe10.3,1x,'Ly=',e10.3,1x,'Lz=',e10.3)

  x_glob = 0.

!  Global mesh
!  ~~~~~~~~~~~
  do idim = 1,3
     nx_dim = nx_glob(idim)
     
     x_glob(       1,idim) = box_min(idim)
     x_glob(nx_dim+1,idim) = box_min(idim) + Lbox(idim)
     
     dx(idim) = (x_glob(nx_dim+1,idim) - x_glob(1,idim))/float(nx_dim)
     do i=2,nx_dim
        x_glob(i,idim) = x_glob(i-1,idim) + dx(idim)
     enddo

  enddo

!  Local mesh
!  ~~~~~~~~~~
  do idim = 1,3
     i1 = 1
     if (icl(2*idim-1) == 0) i1 = 1-nbuf
     i2 = nx(idim)+1
     if (icl(2*idim  ) == 0) i2 = nx(idim)+1+nbuf

     do i=i1,i2
        x(i,idim) = x_glob(x_start(idim)+i-1,idim)
     enddo
  enddo

  ! Variables
  rho1   =  1.e-03   *gramme/cm3
  u1     =  0.       *centimetre/seconde
  T1     =  290      *Kelvin
  
  Tray1  = T1

  ! Marshak test
  rho1  = 1.e-3 *gramme/cm3
  mu    = 1.25e8

  T1    = 300. * Kelvin
  Tray1 = 300. * Kelvin
  
  call energy(rho1,T1,E1,1,1,1,1,1,1)

  if(mype == 0) then
     write(*,*)
     write(*,*) 'Tray1=',Tray1 /Kelvin, 'Kelvin'
  endif

  ! HYDRO initialisation

  if(verbose) write(*,*) 'init hydro'

   rho(1:nx(1),1:nx(2),1:nx(3)       ) = rho1

  rhou(1:nx(1),1:nx(2),1:nx(3),1:ndim) = rho1*u1
     E(1:nx(1),1:nx(2),1:nx(3)       ) =   E1
  do idim=1,ndim
     E(1:nx(1),1:nx(2),1:nx(3)       ) =   E(1:nx(1),1:nx(2),1:nx(3)) + 0.5*rhou(1:nx(1),1:nx(2),1:nx(3),idim)**2/rho(1:nx(1),1:nx(2),1:nx(3))
  enddo

  ! RAYO initialisation

  if(verbose) write(*,*) 'init radiation'

#if RAYO==1
  if(RAY) then
     Eray(1:nx(1),1:nx(2),1:nx(3)       ) = a_r*Tray1**4
     Fray(1:nx(1),1:nx(2),1:nx(3),1:ndim) = 0.
     Tgaz(1:nx(1),1:nx(2),1:nx(3)       ) = T1
  endif
#endif

#if DIF==1
  if(DIFFU) Er_dif(1:nx(1),1:nx(2),1:nx(3)       ) = a_r*Tray1**4
#endif

  ! Boundary conditions
  
   rho_l = rho1
  rhou_l = 0.
     E_l =   E1

#if RAYO==1
     if(RAY) then
        Er_l = a_r*Tray1**4 ; T_l = T1
        Fr_l = 0.
     endif
#endif
#if DIF==1
     if(DIFFU) then
        Er_dif_l = a_r*Tray1**4 ; T_l = T1
     endif
#endif

   !*******************
   ! Marshak
   T_s  = 1000. * Kelvin
   Tr_s = 1000. * Kelvin

#if RAYO==1
   if(RAY  ) then
      Er_l(1+2*(idir-1)) = a_r*Tr_s**4   ; T_l(1+2*(idir-1)) = T_s
   endif
#endif
#if DIF==1
   if(DIFFU) then
      Er_dif_l(1+2*(idir-1)) = a_r*Tr_s**4   ; T_l(1+2*(idir-1)) = T_s
   endif
#endif
   !*******************

  if(verbose) write(*,*) 'End init'

  return
  
end subroutine INIT

!*******************************************************************************************************************

!     ---------------------
subroutine DIMENSIONS
!     ---------------------
  use unites
  use gammas

  implicit none

  ! code units
  unitM = 1.0E+00 *gramme
  unitL = 1.0E+00 *centimetre
  unitT = 1.0E+00 *seconde
  unitD = unitM/unitL**3

  ! Specific heats ratio Cp/Cv
  gamma = 5.0/3.0
  
  return
  
end subroutine DIMENSIONS
