!     ---------------
SUBROUTINE INIT
!     ---------------
  USE parameters
  USE unites
  USE divers
  USE param_ini
  USE var
  USE gammas
  USE para
  USE cl
  USE geom           , ONLY : x,x_glob,lbox
  USE const

  IMPLICIT NONE

  INTEGER :: i,j,k,idim, i1,i2
  REAL, DIMENSION(1:3   ) :: dx

  REAL    :: P0, xc,yc
  REAL    :: deuxpi!, B0
  REAL    :: Bcx,Bcy,Bcz

  INTEGER :: nx_dim

!----------------------

  IF (verbose) WRITE(*,*) 'debut de init'

  Lbox = 1.
  deuxpi=4.d0*ASIN(1.d0)

  B0=(2.d0*deuxpi)**(-1./2.)
  p0=gamma/(2.d0*deuxpi)


!  temps de dpart
  temps = 0.

!  maillage global
!  ~~~~~~~~~~~~~~
  DO idim = 1,3
     nx_dim = nx_glob(idim)
     
     x_glob(       1,idim) = 0. 
     x_glob(nx_dim+1,idim) = Lbox(idim)
     
     dx(idim) = (x_glob(nx_dim+1,idim) - x_glob(1,idim))/float(nx_dim)
     DO i=2,nx_dim
        x_glob(i,idim) = x_glob(i-1,idim) + dx(idim)
     ENDDO

  ENDDO

!  maillage local
!  ~~~~~~~~~~~~~~
  
  DO idim = 1,3

     i1 = 1
     IF (icl(2*idim-1) == 0) i1 = 1-nbuf
     i2 = nx(idim)+1
     IF (icl(2*idim  ) == 0) i2 = nx(idim)+1+nbuf

     DO i=i1,i2
        x(i,idim) = x_glob(x_start(idim)+i-1,idim)
     ENDDO

  ENDDO


  IF (verbose) WRITE(*,*) '-- 1 --'
 
  mu     =  1.

  rhou = 0.

  DO k=1,nx(3)
  DO j=1,nx(2)+1
  DO i=1,nx(1)+1

     !cell faces
     xc = 0.5*(x(i,1)+x(i+1,1))
     yc = 0.5*(x(j,2)+x(j+1,2))

     !density
     rho(i,j,k) = gamma*p0

     !momentum
     rhou(i,j,k,1) = - rho(i,j,k)*SIN(yc*deuxpi)
     rhou(i,j,k,2) =   rho(i,j,k)*SIN(xc*deuxpi)
     rhou(i,j,k,3) = 0.

     !magnetic field
     B(i,j,k,1) = - B0*SIN(     yc*deuxpi)
     B(i,j,k,2) =   B0*SIN(2.d0*xc*deuxpi)
     B(i,j,k,3) =  0.

  ENDDO
  ENDDO
  ENDDO

  IF (verbose) WRITE(*,*) '-- 2 --'

  DO k=1,nx(3)
  DO j=1,nx(2)
  DO i=1,nx(1)

     !Energy
     BCx = half*( B(i,j,k,1) + B(i+1,j  ,k  ,1) )
     BCy = half*( B(i,j,k,2) + B(i  ,j+1,k  ,2) )
     BCz = B(i,j,k,3)
     E(i,j,k) = p0 / (gamma-1.) + 0.5 * ( &
&                  (rhou(i,j,k,1  )**2 + rhou(i,j,k,2  )**2 + rhou(i,j,k,3  )**2) / rho(i,j,k) &   
&                + (            Bcx**2 +             Bcy**2 +             Bcz**2)     )


  ENDDO
  ENDDO
  ENDDO

  IF (verbose) WRITE(*,*) 'Fin de init'

  RETURN
END SUBROUTINE INIT

!*******************************************************************************************************************

!     ---------------------
subroutine DIMENSIONS
!     ---------------------
  use unites
  use gammas

  implicit none

  ! code units
  unitM = 1.0E+00 *gramme
  unitL = 1.0E+00 *centimetre
  unitT = 1.0E+00 *seconde
  unitD = unitM/unitL**3

  ! Specific heats ratio Cp/Cv
  gamma = 5.0/3.0
  
  return
  
end subroutine DIMENSIONS
