module shock
    character(5) :: shock_type
end module shock

!     ---------------
subroutine INIT
!     ---------------
  use parameters, only:nx,ndim,nx_glob,nbuf
  use unites
  use divers
  use divers_ray , only : rho1,T1,ordre_des_faces_pour_le_ray
  use var
  use varray
  use geom  , only : x,x_glob,Lbox,box_min,box_max
  use cl    , only : icl, rho_l, rhou_l, E_l, T_l
  use para  , only : mype,x_start
  use shock

#if RAYO==1
  use cl_ray, only : Fr_l, Er_l, Tr_l
#endif

#if DIF==1
  use diffusion
  use cl_dif
#endif

  implicit none
  integer :: i,j,k,idim,i1,i2,nx_dim,idir,iloc

  real ,dimension(1:3) :: dx

  real :: Tray1 !rho1 and T1 are already included in module divers
  real :: u1,E1,cs1,Etemp

  real :: rho2,u2,T2,Tray2,E2

  namelist/condinit/rho1,u1,T1,Tray1,Lbox,mu,box_min,idir,shock_type
!----------------------

! Begin time
  temps = 0.

  mu     =  1.
  Tray1  =  1.

  rho1   =  1.2e-03
  T1     =  10.    
  u1     =  200.   

  Lbox = 1.
  box_min=0.

  idir = 1

  shock_type = 'stat'

  ! Reading the namelist (previous values are erased)
  if(mype == 0) write(*,'(a)') 'reading namelist CONDINIT'
  open  (1,file='liste.nml')
  rewind(1                 )
  read  (1,NML=condinit    )
  close (1                 )

  tend   = tend   * seconde
  dt_max = dt_max * seconde

  ! Order of the faces for the radiative boundary conditions
  select case (idir)
  case (1)
     ordre_des_faces_pour_le_ray(1) = 1
     ordre_des_faces_pour_le_ray(2) = 2
     ordre_des_faces_pour_le_ray(3) = 3
     ordre_des_faces_pour_le_ray(4) = 4
     ordre_des_faces_pour_le_ray(5) = 5
     ordre_des_faces_pour_le_ray(6) = 6
  case (2)
     ordre_des_faces_pour_le_ray(1) = 3
     ordre_des_faces_pour_le_ray(2) = 4
     ordre_des_faces_pour_le_ray(3) = 1
     ordre_des_faces_pour_le_ray(4) = 2
     ordre_des_faces_pour_le_ray(5) = 5
     ordre_des_faces_pour_le_ray(6) = 6
  case (3)
     ordre_des_faces_pour_le_ray(1) = 5
     ordre_des_faces_pour_le_ray(2) = 6
     ordre_des_faces_pour_le_ray(3) = 1
     ordre_des_faces_pour_le_ray(4) = 2
     ordre_des_faces_pour_le_ray(5) = 3
     ordre_des_faces_pour_le_ray(6) = 4
  end select

!  Mesh
  Lbox(1:ndim) =  Lbox(1:ndim) * centimetre 

  if( mype == 0 ) then
     write(*,*)
     write(*,*) 'Dimensions of the global simulation (in cells numbers and centimetres):'
     write(*,001) ndim
     write(*,002) nx_glob(1),nx_glob(2),nx_glob(3),Lbox(1)/centimetre,Lbox(2)/centimetre,Lbox(3)/centimetre
     write(*,*)
  endif

001 format('ndim = ',I1)
002 format('nx=',I4,1x,'ny=',I4,1x,'nz=',I4,1x,'Lx=',1pe10.3,1x,'Ly=',e10.3,1x,'Lz=',e10.3)

  x_glob = 0.

!  Global mesh
!  ~~~~~~~~~~~
  do idim = 1,3
     nx_dim = nx_glob(idim)
     
     x_glob(       1,idim) = box_min(idim)
     x_glob(nx_dim+1,idim) = box_min(idim) + Lbox(idim)
     
     dx(idim) = (x_glob(nx_dim+1,idim) - x_glob(1,idim))/float(nx_dim)
     do i=2,nx_dim
        x_glob(i,idim) = x_glob(i-1,idim) + dx(idim)
     enddo

  enddo

!  Local mesh
!  ~~~~~~~~~~
  do idim = 1,3
     i1 = 1
     if (icl(2*idim-1) == 0) i1 = 1-nbuf
     i2 = nx(idim)+1
     if (icl(2*idim  ) == 0) i2 = nx(idim)+1+nbuf

     do i=i1,i2
        x(i,idim) = x_glob(x_start(idim)+i-1,idim)
     enddo
  enddo

!Variables
   rho1   =  rho1 *gramme/cm3
   T1     =  T1   *Kelvin
   u1     =  u1   *kms

   Tray1  = T1

   call energy(rho1,T1,E1,1,1,1,1,1,1)
   call sound_speed(rho1,E1,cs1,1,1,1,1,1,1)

  if(mype == 0) then
     write(*,*)
     write(*,*) 'Tray1=',Tray1 /Kelvin, 'Kelvin'

     write(*,*) 'cs1=',cs1/kms, 'km/s'
     write(*,*) 'Mach1=',u1/cs1
  endif

  if(shock_type .eq. 'stat') then
     ! Rankine-Hugoniot relation
     call rh(rho1,u1,T1,rho2,u2,T2)
     u1 = -u1
     u2 = -u2
     Tray2 = T2  

     call energy(rho2,T2,E2,1,1,1,1,1,1)
  endif

  ! HYDRO initialisation

  if(verbose) write(*,*) 'init hydro'
  
  do k=1,nx(3)
  do j=1,nx(2)
  do i=1,nx(1)
     if(idir==1) iloc=i
     if(idir==2) iloc=j
     if(idir==3) iloc=k

     if(shock_type.eq.'stat') then
        if(x(iloc,idir).lt.box_min(idir)+Lbox(idir)/2) then
           rho(i,j,k       ) = rho2
           rhou(i,j,k,1:ndim) = 0.
           rhou(i,j,k,  idir) = rho2*u2
           E(i,j,k       ) =   E2
           do idim=1,ndim
              E(i,j,k       ) =   E(i,j,k) + 0.5*rhou(i,j,k,idim)**2/rho(i,j,k)
           enddo
#if RAYO==1
           if(RAY) then
              Eray(i,j,k       ) = a_r*Tray2**4
              Fray(i,j,k,1:ndim) = 0.
              Tgaz(i,j,k       ) = T2
           endif
#endif
#if DIF==1
           if(DIFFU) Er_dif(i,j,k) = a_r*Tray2**4
#endif
        else
           rho(i,j,k       ) = rho1
           rhou(i,j,k,1:ndim) = 0.
           rhou(i,j,k,  idir) = rho1*u1
           E(i,j,k       ) =   E1
           do idim=1,ndim
              E(i,j,k       ) =   E(i,j,k) + 0.5*rhou(i,j,k,idim)**2/rho(i,j,k)
           enddo
#if RAYO==1
           if(RAY) then
              Eray(i,j,k       ) = a_r*Tray1**4
              Fray(i,j,k,1:ndim) = 0.
              Tgaz(i,j,k       ) = T1
           endif
#endif
#if DIF==1
           if(DIFFU) Er_dif(i,j,k) = a_r*Tray1**4
#endif

        endif
     else
        rho(i,j,k        ) = rho1
        rhou(i,j,k,1:ndim) = 0.
        rhou(i,j,k,  idir) = rho1*u1
        call energy(rho(i,j,k),T1+0.25*Kelvin*(nx_glob(idir)+1-(iloc+x_start(idir)-1)),Etemp,1,1,1,1,1,1)
        E(i,j,k) = Etemp
        do idim=1,ndim
           E(i,j,k       ) =   E(i,j,k) + 0.5*rhou(i,j,k,idim)**2/rho(i,j,k)
        enddo
#if RAYO==1
        if(RAY) then
           Eray(i,j,k       ) = a_r*(Tray1+0.25*Kelvin*(nx_glob(idir)+1-(iloc+x_start(idir)-1)))**4
           Fray(i,j,k,1:ndim) = 0.
           Tgaz(i,j,k       ) = T1
        endif
#endif
#if DIF==1
           if(DIFFU) Er_dif(i,j,k) = a_r*(Tray1+0.25*Kelvin*(nx_glob(idir)+1-(iloc+x_start(idir)-1)))**4
#endif
     endif

  enddo
  enddo
  enddo

  ! Boundary conditions

   rho_l(     2*(idir-1)+2) = rho1
  rhou_l                    = 0.
  rhou_l(idir,2*(idir-1)+2) = rho1*u1
     E_l(     2*(idir-1)+2) =   E1 + 0.5*rho1*u1*u1

     if (shock_type .eq. 'stat') then
        rho_l(     2*(idir-1)+1) = rho2
        rhou_l(idir,2*(idir-1)+1) = rho2*u2
        E_l(     2*(idir-1)+1) =   E2 + 0.5*rho2*u2*u2
     endif

#if RAYO==1
     if(RAY) then
        Er_l = a_r*Tray1**4 ; T_l = T1
        Fr_l = 0.

        if(shock_type .eq. 'stat') then
           Er_l(2*(idir-1)+1) = a_r*Tray2**4 ; T_l(2*(idir-1)+1) = T2
        endif
    endif
#endif
#if DIF==1
     if(DIFFU) then
        Er_dif_l = a_r*Tray1**4 ; T_l = T1
        if(shock_type .eq. 'stat') then
           Er_dif_l(2*(idir-1)+1) = a_r*Tray2**4 ; T_l(2*(idir-1)+1) = T2
        endif
     endif
#endif

  if(verbose) write(*,*) 'End init'


  return
  
end subroutine INIT

!*******************************************************************************************************************

!     ---------------------
subroutine DIMENSIONS
!     ---------------------
  use unites
  use gammas

  implicit none

  ! code units
  unitM = 1.0E+00 *gramme
  unitL = 1.0E+00 *centimetre
  unitT = 1.0E+00 *seconde
  unitD = unitM/unitL**3

  ! Specific heats ratio Cp/Cv
  gamma = 7.0/5.0
  
  return
  
end subroutine DIMENSIONS
