!     ---------------
subroutine INIT
  !     ---------------
  use parameters
  use unites
  use divers
  use var
  use gammas
  use para
  use cl
  use geom  , only : x,x_glob,lbox
  implicit none

  integer :: i,ii,j,k,idim,i1,i2,iy,iz
  real, dimension(1:3) :: dx

  integer :: nx_dim,idir

  real    :: rho1,u1,P1,E1,By1,Bz1
  real    :: rho2,u2,P2,E2,By2,Bz2
  real    :: Bx,lmin,xx

  namelist/condinit/idir,lmin,Lbox
  !----------------------

  if (verbose) write(*,*) 'Entering init subroutine...'

  temps = 0.

  Lbox=2.5
  lmin=-1.

  ! Reading the namelist (previous values are erased)
  if(mype == 0) write(*,'(a)') 'reading namelist CONDINIT'
  open  (1,file='liste.nml')
  rewind(1                 )
  read  (1,NML=condinit    )
  close (1                 )

  tend   = tend   * seconde
  dt_max = dt_max * seconde

  Lbox = Lbox*centimetre

  !  Global grid
  !  ~~~~~~~~~~~~~~
  do idim = 1,3
     nx_dim = nx_glob(idim)
     x_glob(       1,idim) = lmin
     x_glob(nx_dim+1,idim) = lmin+Lbox(idim)
     dx(idim) = (x_glob(nx_dim+1,idim) - x_glob(1,idim))/float(nx_dim)
     do i=2,nx_dim
        x_glob(i,idim) = x_glob(i-1,idim) + dx(idim)
     enddo
  enddo

  !  Local grid
  !  ~~~~~~~~~~~~~~

  do idim = 1,3
     i1 = 1
     if (icl(2*idim-1) == 0) i1 = 1-nbuf
     i2 = nx(idim)+1
     if (icl(2*idim  ) == 0) i2 = nx(idim)+1+nbuf 
     do i=i1,i2
        x(i,idim) = x_glob(x_start(idim)+i-1,idim)
     enddo
  enddo

  ! Sod
  ! ~~~~~~~~~~
  alpha=3.

  rho1 = 1. ; rho2 = 0.2
  P1   = 1. ; P2   = 0.2  
  u1   = 0. ; u2   = 0.  
  Bx   = 1.
  By1  = 1. ; By2  = cos(alpha)
  Bz1  = 0. ; Bz2  = sin(alpha)
  E1   =  P1/(gamma-1.) + 0.5*rho1*u1*u1 + 0.5*(Bx*Bx+By1*By1+Bz1*Bz1)
  E2   =  P2/(gamma-1.) + 0.5*rho2*u2*u2 + 0.5*(Bx*Bx+By2*By2+Bz2*Bz2)

  dt_max = 1.

  do k=1,nx(3)
     do j=1,nx(2)
        do i=1,nx(1)

           if(idir==1) then
              ii=i ; iy=2 ; iz=3
           else if(idir==2) then
              ii=j ; iy=3 ; iz=1
           else if(idir==3) then
              ii=k ; iy=1 ; iz=2
           endif
           xx = 0.5*(x(ii,idir)+x(ii+1,idir))

           if (xx < 0.) then
              rhou(i,j,k,idir) = rho1*u1
              E   (i,j,k  )    = E1
              rho (i,j,k  )    = rho1
              B   (i,j,k,idir) = Bx
              B   (i,j,k,iy  ) = By1
              B   (i,j,k,iz  ) = Bz1
           else
              rhou(i,j,k,idir) = rho2*u2 
              E   (i,j,k  )    = E2
              rho (i,j,k  )    = rho2
              B   (i,j,k,idir) = Bx
              B   (i,j,k,iy  ) = By2
              B   (i,j,k,iz  ) = Bz2
           endif
        enddo
     enddo
  enddo

  if (verbose) write(*,*) 'Init subroutine finished.'

  return
  
end subroutine INIT

!*******************************************************************************************************************

!     ---------------------
subroutine DIMENSIONS
!     ---------------------
  use unites
  use gammas

  implicit none

  ! code units
  unitM = 1.0E+00 *gramme
  unitL = 1.0E+00 *centimetre
  unitT = 1.0E+00 *seconde
  unitD = unitM/unitL**3

  ! Specific heats ratio Cp/Cv
  gamma = 5.0/3.0
  
  return
  
end subroutine DIMENSIONS
