pro shock,shock_type,dir=dir, ps=ps

  if not keyword_set(dir) then dir='x'

  c  =2.997925e+10;* centimetre/seconde 
  a_r=7.56464e-15 ;* erg/centimetre**3/degre**4 

  Lbox_min=0.

  loadct,0
  tek_color

  ;Read heracles solution
  RESOLVE_ROUTINE,'rd',/COMPILE_FULL_FILE
  rd,1,var1

  if keyword_set(ps) then begin
     set_plot, 'ps' ; Set plotting to PostScript.
     device, /color, bits=8, xsize=12, ysize=8, /inches, filename=shock_type+'_shock.ps' ; Set the filename.
  endif

  if(shock_type eq 'stat') then begin
     !p.multi=[0,2,2]

     CASE dir OF  
        'x': begin
           x0    = (max(var1.mesh.xc)+min(var1.mesh.xc))/2.
           x     = var1.mesh.xc
           
           rho     = var1.rho(*,0,0)
           u       = var1.u(*,0,0)
           t       = var1.t(*,0,0)
           tr      = (var1.er(*,0,0)/a_r)^0.25
           fr      = var1.fr(*,0,0,0)
        end
        'y': begin
           x0=(max(var1.mesh.yc)+min(var1.mesh.yc))/2.
           x     = var1.mesh.yc
           
           rho     = var1.rho(0,*,0)
           u       = var1.u(0,*,0)
           t       = var1.t(0,*,0)
           tr      = (var1.er(0,*,0)/a_r)^0.25
           fr      = var1.fr(0,*,0,1)
        end
        'z': begin
           x0=(max(var1.mesh.zc)+min(var1.mesh.zc))/2.
           x     = var1.mesh.zc
           
           rho     = var1.rho(0,0,*)
           u       = var1.u(0,0,*)
           t       = var1.t(0,0,*)
           tr      = (var1.er(0,0,*)/a_r)^0.25
           fr      = var1.fr(0,0,*,2)
        end
     ENDCASE

     plot,x-x0,rho    ,xtitle=dir+' (cm)',ytitle='Density (g/cm!u3!n)',xr=[-0.5,0.5],/xs
     plot,x-x0,u*1.e-5,xtitle=dir+' (cm)',ytitle='Velocity (km/s)',xr=[-0.5,0.5],/xs
     plot,x-x0,t      ,xtitle=dir+' (cm)',ytitle='Temperature (K)',xr=[-0.5,0.5],/xs
    oplot,x-x0,tr
     plot,x-x0,fr     ,xtitle=dir+' (cm)',ytitle='Radiative flux (erg/s/cm!u2!n)',xr=[-0.5,0.5],/xs

     !p.multi=0
  endif else begin
     Lbox_min=8.e11
     if(shock_type eq 'sub') then begin
        v=-6.e-5    ; in units of 1.e5km/s
        xr=[0.5,5]
        yr=[0,1200]
     endif else if(shock_type eq 'super') then begin
        v=-20.e-5   ; in units of 1.e5km/s
        xr=[0.5,9]
        yr=[0,6000]
     endif

     rd,2,var2
     rd,3,var3

     CASE dir OF  
        'x': begin
           ; x in units of 1.e5km
           x1    =(var1.mesh.xc-Lbox_min)/1.e10
           x2    =(var2.mesh.xc-Lbox_min)/1.e10
           x3    =(var3.mesh.xc-Lbox_min)/1.e10
           
           t1=var1.t(*,0,0)
           t2=var2.t(*,0,0)
           t3=var3.t(*,0,0)
           tr1=(var1.er(*,0,0)/a_r)^0.25
           tr2=(var2.er(*,0,0)/a_r)^0.25
           tr3=(var3.er(*,0,0)/a_r)^0.25
        end
        'y': begin
           ; x in units of 1.e5km
           x1    =(var1.mesh.yc-Lbox_min)/1.e10
           x2    =(var2.mesh.yc-Lbox_min)/1.e10
           x3    =(var3.mesh.yc-Lbox_min)/1.e10
           
           t1=var1.t(0,*,0)
           t2=var2.t(0,*,0)
           t3=var3.t(0,*,0)
           tr1=(var1.er(0,*,0)/a_r)^0.25
           tr2=(var2.er(0,*,0)/a_r)^0.25
           tr3=(var3.er(0,*,0)/a_r)^0.25
        end
        'z': begin
           ; x in units of 1.e5km
           x1    =(var1.mesh.zc-Lbox_min)/1.e10
           x2    =(var2.mesh.zc-Lbox_min)/1.e10
           x3    =(var3.mesh.zc-Lbox_min)/1.e10
           
           t1=var1.t(0,0,*)
           t2=var2.t(0,0,*)
           t3=var3.t(0,0,*)
           tr1=(var1.er(0,0,*)/a_r)^0.25
           tr2=(var2.er(0,0,*)/a_r)^0.25
           tr3=(var3.er(0,0,*)/a_r)^0.25
        end
     ENDCASE

      plot,x1-v*var1.time,t1,xr=xr,yr=yr,/xs,/ys,xtitle=dir+'!di !n(10!u5!n km)',ytitle='Temperature (K)',psym=psym
     oplot,x1-v*var1.time,tr1,linestyle=2
     
     oplot,x2-v*var2.time,t2,psym=psym
     oplot,x2-v*var2.time,tr2,linestyle=2
     
     oplot,x3-v*var3.time,t3,psym=psym
     oplot,x3-v*var3.time,tr3,linestyle=2

  endelse

  if keyword_set(ps) then begin
     device, /close ; Close the file. 
     set_plot,'x'
  endif
  
end
