function rd_ascii,filename,ncol=ncol,skipline=skipline

if not(keyword_set(ncol)) then ncol=2
if not(keyword_set(skipline)) then skipline=0
nlines=file_lines(filename)
data=fltarr(ncol,nlines-skipline)

toto=''

openr,1,filename
if (skipline gt 0) then begin
    for iskip=0,skipline-1 do readf,1,toto
endif
readf,1,data
close,1

return,data

end
;--------------------------------------------------------------------
pro tubemhd,dir,isotherm=isotherm, ps=ps

;Read heracles solution
RESOLVE_ROUTINE,'rd',/COMPILE_FULL_FILE
rd,1,var

;Dir is the direction of the Sod test
CASE dir OF  
   'x': begin
      x=var.mesh.xc   
      r=var.rho(*,0,0)
      b=var.b(*,0,0,1)
      end
   'y': begin
      x=var.mesh.yc   
      r=var.rho(0,*,0)
      b=var.b(0,*,0,2)
      end
   'z': begin
      x=var.mesh.zc   
      r=var.rho(0,0,*)
      b=var.b(0,0,*,0)
      end
ENDCASE  

;Compute analytic solution
data=rd_ascii('../ref/tubemhd_exact.dat',ncol=9)
x_th=data(0,*)
r_th=data(1,*)
b_th=data(6,*)

tek_color
!p.charsize=1.5

if keyword_set(ps) then begin
  set_plot, 'ps' ; Set plotting to PostScript.
  device, /color, bits=8, xsize=12, ysize=4, /inches, filename='tubemhd'+dir+'.ps' ; Set the filename.
  !p.multi = [0, 2, 1]
endif

;Plot density
if not keyword_set(ps) then window,0
 plot,x   ,r   ,psym=4,xtitle=dir,title='Density'
oplot,x_th,r_th,thick=3,color=2

;Plot by
if not keyword_set(ps) then window,1
 plot,x   ,b   ,psym=4,xtitle=dir,title='By'
oplot,x_th,b_th,thick=3,color=2

if keyword_set(ps) then begin
  device, /close ; Close the file. 
  set_plot,'x'
endif

return
end


function get_analytic_rho,x_e,Nth
r_e=fltarr(Nth) & r_e(*)=1.
for i=0,Nth-1 do begin
   if ((x_e(i) ge 1.-0.7082423) and (x_e(i) le 1.-0.3022800)) then $
      r_e(i)=1.-(x_e(i)-1.+0.7082423)/(0.7082423-0.3022800)*(1.-0.6452294)
   if ((x_e(i) ge 1.-0.3022800) and (x_e(i) le 1.-0.2412073)) then r_e(i)=0.6452294
   if ((x_e(i) ge 1.-0.2412073) and (x_e(i) le 1.-0.1800155)) then $
      r_e(i)=0.8410898-(x_e(i)-1.+0.2412073)/(0.2412073-0.1800155)*(0.8410898-0.7538870)
   if ((x_e(i) ge 1.-0.1800155) and (x_e(i) le 1.+0.2036784)) then r_e(i)=0.7538870
   if ((x_e(i) ge 1.+0.2036784) and (x_e(i) le 1.+0.5537568)) then r_e(i)=0.3653074
   if ((x_e(i) ge 1.+0.5537568) and (x_e(i) le 1.+1.0438319)) then r_e(i)=0.1728151
   if ((x_e(i) ge 1.+1.0438319) and (x_e(i) le 1.+1.3207457)) then $
      r_e(i)=0.1728151+(x_e(i)-1.-1.0438319)/(1.3207457-1.0438319)*(0.2-0.1728151)
   if (x_e(i) ge 1.+1.3207457) then r_e(i)=0.2
endfor
return,r_e
end

function get_analytic_by,x_e,Nth
b_e=fltarr(Nth) & b_e(*)=1.
for i=0,Nth-1 do begin
   if ((x_e(i) ge 1.-0.7082423) and (x_e(i) le 1.-0.3022800)) then $
      b_e(i)=1.-(x_e(i)-1.+0.7082423)/(0.7082423-0.3022800)*(1.-0.3986243)
   if ((x_e(i) ge 1.-0.3022800) and (x_e(i) le 1.-0.2412073)) then b_e(i)=0.3986243
   if ((x_e(i) ge 1.-0.2412073) and (x_e(i) le 1.-0.1800155)) then $
      b_e(i)=-0.2825269-(x_e(i)-1.+0.2412073)/(0.2412073-0.1800155)*(-0.2825269+0.4140819)
   if ((x_e(i) ge 1.-0.1800155) and (x_e(i) le 1.+0.5537568)) then b_e(i)=-0.4140819
   if ((x_e(i) ge 1.+0.5537568) and (x_e(i) le 1.+1.0438319)) then b_e(i)=-0.7300059
   if ((x_e(i) ge 1.+0.5537568) and (x_e(i) le 1.+1.0438319)) then b_e(i)=-0.7300059
   if ((x_e(i) ge 1.+1.0438319) and (x_e(i) le 1.+1.3207457)) then $
      b_e(i)=-0.7300059+(x_e(i)-1.-1.0438319)/(1.3207457-1.0438319)*(-1.+0.7300059)
   if (x_e(i) ge 1.+1.3207457) then b_e(i)=-1.
endfor
return,b_e
end
